# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

RETRO_TS_PY_METADATA = Metadata(
    id="3a8f942af6b333e80cee0dd94a9b9a446d4fee85.boutiques",
    name="RetroTS.py",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class RetroTsPyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `retro_ts_py(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType | None
    """Output file containing respiratory and cardiac regressors"""


def retro_ts_py(
    num_slices: float,
    volume_tr: float,
    resp_file: InputPathType | None = None,
    card_file: InputPathType | None = None,
    phys_fs: float | None = None,
    phys_file: InputPathType | None = None,
    phys_json: InputPathType | None = None,
    prefix: str | None = None,
    rvt_shifts: str | None = None,
    rvt_out: bool = False,
    resp_cutoff_freq: float | None = None,
    cardiac_cutoff_freq: float | None = None,
    cardiac_out: bool = False,
    respiration_out: bool = False,
    interp_style: str | None = None,
    fir_order: float | None = None,
    quiet: bool = False,
    demo: bool = False,
    show_graphs: bool = False,
    debug: bool = False,
    slice_offset: str | None = None,
    slice_major: float | None = None,
    slice_order: str | None = None,
    zero_phase_offset: bool = False,
    legacy_transform: float | None = None,
    runner: Runner | None = None,
) -> RetroTsPyOutputs:
    """
    Creates slice-based regressors for regressing out components of heart rate,
    respiration, and respiration volume per time using independent data files or
    BIDS formatted files.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/RetroTS.py.html
    
    Args:
        num_slices: Number of slices.
        volume_tr: Volume TR in seconds.
        resp_file: Respiration data file.
        card_file: Cardiac data file.
        phys_fs: Physiological signal sampling frequency in Hz.
        phys_file: BIDS formatted physio file in tab-separated format, can be\
            gzipped.
        phys_json: BIDS formatted physio metadata json file. If not specified,\
            the json corresponding to the phys_file will be loaded.
        prefix: Prefix of output file.
        rvt_shifts: Vector of shifts in seconds of RVT signal. (default is\
            [0:5:20]).
        rvt_out: Flag for writing RVT regressors (default is 1).
        resp_cutoff_freq: Cut-off frequency in Hz for respiratory lowpass\
            filter (default 3 Hz).
        cardiac_cutoff_freq: Cut-off frequency in Hz for cardiac lowpass filter\
            (default 3 Hz).
        cardiac_out: Flag for writing Cardiac regressors (default is 1).
        respiration_out: Flag for writing Respiratory regressors (default is 1).
        interp_style: Resampling kernel (default is 'linear').
        fir_order: Order of FIR filter (default is 40).
        quiet: Show talkative progress as the program runs (default is 1).
        demo: Run demonstration of RetroTS (default is 0).
        show_graphs: Show graphs (default is unset; set with any parameter to\
            view).
        debug: Drop into pdb upon an exception (default is False).
        slice_offset: Vector of slice acquisition time offsets in seconds\
            (default is equivalent of alt+z).
        slice_major: Unknown parameter (default is 1).
        slice_order: Slice timing information in seconds. (default is alt+z).
        zero_phase_offset: Zero phase offset flag.
        legacy_transform: Specify the version of the original Matlab code's\
            transformation (default is 0).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `RetroTsPyOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(RETRO_TS_PY_METADATA)
    cargs = []
    cargs.append("python")
    cargs.append("RetroTS.py")
    if resp_file is not None:
        cargs.extend([
            "-r",
            execution.input_file(resp_file)
        ])
    if card_file is not None:
        cargs.extend([
            "-c",
            execution.input_file(card_file)
        ])
    if phys_fs is not None:
        cargs.extend([
            "-p",
            str(phys_fs)
        ])
    cargs.extend([
        "-n",
        str(num_slices)
    ])
    cargs.extend([
        "-v",
        str(volume_tr)
    ])
    if phys_file is not None:
        cargs.extend([
            "-phys_file",
            execution.input_file(phys_file)
        ])
    if phys_json is not None:
        cargs.extend([
            "-phys_json",
            execution.input_file(phys_json)
        ])
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    if rvt_shifts is not None:
        cargs.extend([
            "-rvt_shifts",
            rvt_shifts
        ])
    if rvt_out:
        cargs.append("-rvt_out")
    if resp_cutoff_freq is not None:
        cargs.extend([
            "-respiration_cutoff_frequency",
            str(resp_cutoff_freq)
        ])
    if cardiac_cutoff_freq is not None:
        cargs.extend([
            "-cardiac_cutoff_frequency",
            str(cardiac_cutoff_freq)
        ])
    if cardiac_out:
        cargs.append("-cardiac_out")
    if respiration_out:
        cargs.append("-respiration_out")
    if interp_style is not None:
        cargs.extend([
            "-interpolation_style",
            interp_style
        ])
    if fir_order is not None:
        cargs.extend([
            "-fir_order",
            str(fir_order)
        ])
    if quiet:
        cargs.append("-quiet")
    if demo:
        cargs.append("-demo")
    if show_graphs:
        cargs.append("-show_graphs")
    if debug:
        cargs.append("-debug")
    if slice_offset is not None:
        cargs.extend([
            "-slice_offset",
            slice_offset
        ])
    if slice_major is not None:
        cargs.extend([
            "-slice_major",
            str(slice_major)
        ])
    if slice_order is not None:
        cargs.extend([
            "-slice_order",
            slice_order
        ])
    if zero_phase_offset:
        cargs.append("-zero_phase_offset")
    if legacy_transform is not None:
        cargs.extend([
            "-legacy_transform",
            str(legacy_transform)
        ])
    ret = RetroTsPyOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(prefix + ".slibase.1D") if (prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "RETRO_TS_PY_METADATA",
    "RetroTsPyOutputs",
    "retro_ts_py",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CIFTI_MERGE_DENSE_METADATA = Metadata(
    id="00f701d21942a962666b6b3ef46f75a7ac89d730.boutiques",
    name="cifti-merge-dense",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class CiftiMergeDenseCifti:
    """
    specify an input cifti file.
    """
    cifti_in: InputPathType
    """a cifti file to merge"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-cifti")
        cargs.append(execution.input_file(self.cifti_in))
        return cargs


class CiftiMergeDenseOutputs(typing.NamedTuple):
    """
    Output object returned when calling `cifti_merge_dense(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_merge_dense(
    direction: str,
    cifti_out: str,
    opt_label_collision_action: str | None = None,
    cifti: list[CiftiMergeDenseCifti] | None = None,
    runner: Runner | None = None,
) -> CiftiMergeDenseOutputs:
    """
    Merge cifti files along dense dimension.
    
    The input cifti files must have matching mappings along the direction not
    specified, and the mapping along the specified direction must be brain
    models.
    
    Author: Washington University School of Medicin
    
    Args:
        direction: which dimension to merge along, ROW or COLUMN.
        cifti_out: the output cifti file.
        opt_label_collision_action: how to handle conflicts between label keys:\
            'ERROR', 'FIRST', or 'LEGACY', default 'ERROR', use 'LEGACY' to match\
            v1.4.2 and earlier.
        cifti: specify an input cifti file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiMergeDenseOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_MERGE_DENSE_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-merge-dense")
    cargs.append(direction)
    cargs.append(cifti_out)
    if opt_label_collision_action is not None:
        cargs.extend([
            "-label-collision",
            opt_label_collision_action
        ])
    if cifti is not None:
        cargs.extend([a for c in [s.run(execution) for s in cifti] for a in c])
    ret = CiftiMergeDenseOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(cifti_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CIFTI_MERGE_DENSE_METADATA",
    "CiftiMergeDenseCifti",
    "CiftiMergeDenseOutputs",
    "cifti_merge_dense",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_DESPIKE_METADATA = Metadata(
    id="3963b4a51ef25ad42794c6077f4c3e3027cc9721.boutiques",
    name="3dDespike",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dDespikeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_despike(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_file: OutputPathType
    """Output file."""


def v_3d_despike(
    in_file: InputPathType,
    runner: Runner | None = None,
) -> V3dDespikeOutputs:
    """
    Removes 'spikes' from the 3D+time input dataset and writes a new dataset with
    the spike values replaced by something more pleasing to the eye.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dDespike.html
    
    Args:
        in_file: Input file to 3ddespike.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dDespikeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_DESPIKE_METADATA)
    cargs = []
    cargs.append("3dDespike")
    cargs.append(execution.input_file(in_file))
    cargs.append("[OUT_FILE]")
    ret = V3dDespikeOutputs(
        root=execution.output_file("."),
        out_file=execution.output_file(pathlib.Path(in_file).name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dDespikeOutputs",
    "V_3D_DESPIKE_METADATA",
    "v_3d_despike",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_SEG_METADATA = Metadata(
    id="21efe6308e4d6e96528d68fccee473158117e121.boutiques",
    name="3dSeg",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dSegOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_seg(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    segmented_volume: OutputPathType | None
    """Segmented brain volume output"""
    bias_field: OutputPathType | None
    """Bias field estimate output"""
    classified_volume: OutputPathType | None
    """Classified volume output"""


def v_3d_seg(
    anat: InputPathType,
    mask: str | None = None,
    blur_meth: str | None = None,
    bias_fwhm: float | None = None,
    classes: str | None = None,
    bmrf: float | None = None,
    bias_classes: str | None = None,
    prefix: str | None = None,
    overwrite: bool = False,
    debug: float | None = None,
    mixfrac: str | None = None,
    mixfloor: float | None = None,
    gold: InputPathType | None = None,
    gold_bias: InputPathType | None = None,
    main_n: float | None = None,
    cset: InputPathType | None = None,
    labeltable: InputPathType | None = None,
    vox_debug: str | None = None,
    vox_debug_file: str | None = None,
    runner: Runner | None = None,
) -> V3dSegOutputs:
    """
    Segments brain volumes into tissue classes with optional global and voxelwise
    priors.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dSeg.html
    
    Args:
        anat: Volume to segment.
        mask: Mask of the volume to be segmented. Can be a dataset or 'AUTO' to\
            use AFNI's automask function.
        blur_meth: Blurring method for bias field estimation. Options: BFT,\
            BIM, BNN, LSB. Default: BFT.
        bias_fwhm: The amount of blurring used when estimating the field bias.\
            Default: 25.0.
        classes: String of class labels separated by semicolons. Default: 'CSF;\
            GM; WM'.
        bmrf: Weighting factor controlling spatial homogeneity of\
            classifications. Default: 0.0.
        bias_classes: Classes that contribute to the estimation of the bias\
            field. Default: 'GM; WM'.
        prefix: Prefix for all output volume.
        overwrite: Automatically overwrite existing files with the same prefix.
        debug: Set debug level (0, 1, or 2).
        mixfrac: Volume-wide mixing fractions for initialization. Options: '0.1\
            0.45 0.45', 'UNI', 'AVG152_BRAIN_MASK', 'IGNORE'. Default: UNI.
        mixfloor: Set the minimum value for any class's mixing fraction.\
            Default: 0.0001.
        gold: Goldstandard segmentation volume for comparison.
        gold_bias: Goldstandard bias volume for comparison.
        main_n: Number of iterations to perform. Default: 5.
        cset: Initial classification. Uses 3dkmean's engine if not provided.
        labeltable: Label table containing integer keys and corresponding\
            labels.
        vox_debug: 1D index of voxel to debug or 3D voxel indices.
        vox_debug_file: File in which debug information is output, use '-' for\
            stdout, '+' for stderr.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dSegOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_SEG_METADATA)
    cargs = []
    cargs.append("3dSeg")
    cargs.append("-anat")
    cargs.append(execution.input_file(anat))
    cargs.append("-mask")
    if mask is not None:
        cargs.append(mask)
    cargs.append("-blur_meth")
    if blur_meth is not None:
        cargs.extend([
            "-blur_meth",
            blur_meth
        ])
    cargs.append("-bias_fwhm")
    if bias_fwhm is not None:
        cargs.extend([
            "-bias_fwhm",
            str(bias_fwhm)
        ])
    cargs.append("-classes")
    if classes is not None:
        cargs.extend([
            "-classes",
            classes
        ])
    cargs.append("-Bmrf")
    if bmrf is not None:
        cargs.extend([
            "-Bmrf",
            str(bmrf)
        ])
    cargs.append("-bias_classes")
    if bias_classes is not None:
        cargs.extend([
            "-bias_classes",
            bias_classes
        ])
    cargs.append("-prefix")
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    cargs.append("-overwrite")
    if overwrite:
        cargs.append("-overwrite")
    cargs.append("-debug")
    if debug is not None:
        cargs.extend([
            "-debug",
            str(debug)
        ])
    cargs.append("-mixfrac")
    if mixfrac is not None:
        cargs.extend([
            "-mixfrac",
            mixfrac
        ])
    cargs.append("-mixfloor")
    if mixfloor is not None:
        cargs.extend([
            "-mixfloor",
            str(mixfloor)
        ])
    cargs.append("-gold")
    if gold is not None:
        cargs.extend([
            "-gold",
            execution.input_file(gold)
        ])
    cargs.append("-gold_bias")
    if gold_bias is not None:
        cargs.extend([
            "-gold_bias",
            execution.input_file(gold_bias)
        ])
    cargs.append("-main_N")
    if main_n is not None:
        cargs.extend([
            "-main_N",
            str(main_n)
        ])
    cargs.append("-cset")
    if cset is not None:
        cargs.extend([
            "-cset",
            execution.input_file(cset)
        ])
    cargs.append("-labeltable")
    if labeltable is not None:
        cargs.extend([
            "-labeltable",
            execution.input_file(labeltable)
        ])
    cargs.append("-vox_debug")
    if vox_debug is not None:
        cargs.extend([
            "-vox_debug",
            vox_debug
        ])
    cargs.append("-vox_debug_file")
    if vox_debug_file is not None:
        cargs.extend([
            "-vox_debug_file",
            vox_debug_file
        ])
    ret = V3dSegOutputs(
        root=execution.output_file("."),
        segmented_volume=execution.output_file(prefix + "_Segsy+orig.HEAD") if (prefix is not None) else None,
        bias_field=execution.output_file(prefix + "_BiasField+orig.HEAD") if (prefix is not None) else None,
        classified_volume=execution.output_file(prefix + "_Classes+orig.HEAD") if (prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dSegOutputs",
    "V_3D_SEG_METADATA",
    "v_3d_seg",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3DDELAY_METADATA = Metadata(
    id="7507c9110b505056676cecc587845fd4335d9ef8.boutiques",
    name="3ddelay",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3ddelayOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3ddelay(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_brick: OutputPathType | None
    """Primary output results Brick for Delay"""
    output_asc: OutputPathType | None
    """Output ASCII file for results"""
    output_asc_log: OutputPathType | None
    """Log file containing parameter settings and warnings"""
    output_asc_ts: OutputPathType | None
    """Output ASCII file with time series"""


def v_3ddelay(
    input_file: InputPathType,
    reference_file: InputPathType,
    sampling_freq: float,
    stim_period: float,
    prefix: str | None = None,
    polort: float | None = None,
    nodtrnd: bool = False,
    units_seconds: bool = False,
    units_degrees: bool = False,
    units_radians: bool = False,
    phzwrp: bool = False,
    nophzwrp: bool = False,
    phzreverse: bool = False,
    phzscale: float | None = None,
    bias: bool = False,
    nobias: bool = False,
    dsamp: bool = False,
    nodsamp: bool = False,
    mask: InputPathType | None = None,
    nfirst: float | None = None,
    nlast: float | None = None,
    co: float | None = None,
    asc: str | None = None,
    ascts: str | None = None,
    runner: Runner | None = None,
) -> V3ddelayOutputs:
    """
    Estimates the time delay between each voxel time series in a 3D+time dataset and
    a reference time series.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3ddelay.html
    
    Args:
        input_file: Filename of the input 3D+time dataset.
        reference_file: Input ideal time series file name.
        sampling_freq: Sampling frequency in Hz. of data time series (1/TR).
        stim_period: Stimulus period in seconds. Set to 0 if stimulus is not\
            periodic.
        prefix: The prefix for the results Brick.
        polort: Detrend input time series with polynomial of specified order.\
            Default is -1 for auto selection.
        nodtrnd: Remove only the mean (equivalent to polort 0).
        units_seconds: Units for delay estimates in seconds.
        units_degrees: Units for delay estimates in degrees. Requires Tstim > 0.
        units_radians: Units for delay estimates in radians. Requires Tstim > 0.
        phzwrp: Wrap delay (or phase) values.
        nophzwrp: Do not wrap phase (default).
        phzreverse: Reverse phase such that phase -> (T-phase).
        phzscale: Scale phase: phase -> phase*SC (default no scaling).
        bias: Do not correct for the bias in the estimates.
        nobias: Correct for the bias in the estimates (default).
        dsamp: Correct for slice timing differences (default).
        nodsamp: Do not correct for slice timing differences.
        mask: Filename of mask dataset. Only voxels with non-zero values in the\
            mask will be considered.
        nfirst: Number of first dataset image to use in the delay estimate.
        nlast: Number of last dataset image to use in the delay estimate.
        co: Cross Correlation Coefficient threshold value to limit ascii output.
        asc: Write the results to an ascii file for voxels with cross\
            correlation coefficients larger than CCT.
        ascts: Write the results and time series to an ascii file for voxels\
            with cross correlation coefficients larger than CCT.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3ddelayOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3DDELAY_METADATA)
    cargs = []
    cargs.append("3ddelay")
    cargs.append(execution.input_file(input_file))
    cargs.append(execution.input_file(reference_file))
    cargs.extend([
        "-fs",
        str(sampling_freq)
    ])
    cargs.extend([
        "-T",
        str(stim_period)
    ])
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    if polort is not None:
        cargs.extend([
            "-polort",
            str(polort)
        ])
    if nodtrnd:
        cargs.append("-nodtrnd")
    if units_seconds:
        cargs.append("-uS")
    if units_degrees:
        cargs.append("-uD")
    if units_radians:
        cargs.append("-uR")
    if phzwrp:
        cargs.append("-phzwrp")
    if nophzwrp:
        cargs.append("-nophzwrp")
    if phzreverse:
        cargs.append("-phzreverse")
    if phzscale is not None:
        cargs.extend([
            "-phzscale",
            str(phzscale)
        ])
    if bias:
        cargs.append("-bias")
    if nobias:
        cargs.append("-nobias")
    if dsamp:
        cargs.append("-dsamp")
    if nodsamp:
        cargs.append("-nodsamp")
    if mask is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask)
        ])
    if nfirst is not None:
        cargs.extend([
            "-nfirst",
            str(nfirst)
        ])
    if nlast is not None:
        cargs.extend([
            "-nlast",
            str(nlast)
        ])
    if co is not None:
        cargs.extend([
            "-co",
            str(co)
        ])
    if asc is not None:
        cargs.extend([
            "-asc",
            asc
        ])
    if ascts is not None:
        cargs.extend([
            "-ascts",
            ascts
        ])
    ret = V3ddelayOutputs(
        root=execution.output_file("."),
        output_brick=execution.output_file(prefix + ".DEL+orig.BRIK") if (prefix is not None) else None,
        output_asc=execution.output_file(prefix + ".ASC") if (prefix is not None) else None,
        output_asc_log=execution.output_file(prefix + ".ASC.log") if (prefix is not None) else None,
        output_asc_ts=execution.output_file(prefix + ".ASC.ts") if (prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3ddelayOutputs",
    "V_3DDELAY_METADATA",
    "v_3ddelay",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CUTOFFCALC_METADATA = Metadata(
    id="9714ed35004136e0a3b4c57d222df3ed14bb182c.boutiques",
    name="cutoffcalc",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class CutoffcalcOutputs(typing.NamedTuple):
    """
    Output object returned when calling `cutoffcalc(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    example_output: OutputPathType
    """Filtered output file if example sigma is provided"""


def cutoffcalc(
    input_design: InputPathType,
    threshold: float | None = 0.9,
    tr: float | None = 3.0,
    lower_limit: float | None = 90.0,
    example_sigma: float | None = None,
    verbose_flag: bool = False,
    debug_flag: bool = False,
    runner: Runner | None = None,
) -> CutoffcalcOutputs:
    """
    Calculates the minimal period for the highpass filter that still preserves a
    specified amount of variance in all the design matrix regressors.
    
    Author: University of Oxford (Mark Jenkinson and Matthew Webster)
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        input_design: Input design matrix (e.g. design.mat).
        threshold: Threshold for retained variance (default=0.9).
        tr: Time between successive data points (default=3.0s).
        lower_limit: Lower limit on period due to autocorr estimation\
            (default=90s).
        example_sigma: Example sigma (in sec) to produce output called\
            example_filt.mtx.
        verbose_flag: Switch on diagnostic messages.
        debug_flag: Switch on debugging messages.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CutoffcalcOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CUTOFFCALC_METADATA)
    cargs = []
    cargs.append("cutoffcalc")
    cargs.append("-i " + execution.input_file(input_design))
    if threshold is not None:
        cargs.append("-t " + str(threshold))
    if tr is not None:
        cargs.append("--tr " + str(tr))
    if lower_limit is not None:
        cargs.append("--limit " + str(lower_limit))
    if example_sigma is not None:
        cargs.append("--example_sig " + str(example_sigma))
    if verbose_flag:
        cargs.append("-v")
    if debug_flag:
        cargs.append("--debug")
    ret = CutoffcalcOutputs(
        root=execution.output_file("."),
        example_output=execution.output_file("example_filt.mtx"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CUTOFFCALC_METADATA",
    "CutoffcalcOutputs",
    "cutoffcalc",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_1D_RPLOT_METADATA = Metadata(
    id="69347b14900f09196c57788d82a5341a202ee054.boutiques",
    name="1dRplot",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V1dRplotOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_1d_rplot(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_plot: OutputPathType
    """Output plot file"""


def v_1d_rplot(
    input_file: InputPathType,
    runner: Runner | None = None,
) -> V1dRplotOutputs:
    """
    Program for plotting a 1D file.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/1dRplot.html
    
    Args:
        input_file: Input 1D file to plot.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V1dRplotOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_1D_RPLOT_METADATA)
    cargs = []
    cargs.append("1dRplot")
    cargs.extend([
        "-input",
        execution.input_file(input_file)
    ])
    cargs.append("[OPTIONS]")
    ret = V1dRplotOutputs(
        root=execution.output_file("."),
        output_plot=execution.output_file("[PREFIX]*.jpg"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V1dRplotOutputs",
    "V_1D_RPLOT_METADATA",
    "v_1d_rplot",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__AFNI_R_PACKAGE_INSTALL_METADATA = Metadata(
    id="d2742cfafc4d7add5aa17b9cf162572247208a8a.boutiques",
    name="@afni_R_package_install",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VAfniRPackageInstallOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__afni_r_package_install(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    install_log: OutputPathType
    """A log file listing all installed R packages."""


def v__afni_r_package_install(
    afni_2: bool = False,
    shiny: bool = False,
    bayes_view: bool = False,
    circos: bool = False,
    custom_packages: str | None = None,
    mirror: str | None = None,
    help_: bool = False,
    runner: Runner | None = None,
) -> VAfniRPackageInstallOutputs:
    """
    Helper script to install R packages for various afni-ish purposes.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/@afni_R_package_install.html
    
    Args:
        afni_2: Install AFNI related R packages.
        shiny: Install AFNI related shiny app packages.
        bayes_view: Install R packages for bayes_view.
        circos: Install OmicCircos for FATCAT_matplot.
        custom_packages: Install custom R packages (space-separated list). Must\
            start and end with double quotes.
        mirror: Set the CRAN mirror to a custom URL.
        help_: Show help message.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VAfniRPackageInstallOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__AFNI_R_PACKAGE_INSTALL_METADATA)
    cargs = []
    cargs.append("@afni_R_package_install")
    if afni_2:
        cargs.append("-afni")
    if shiny:
        cargs.append("-shiny")
    if bayes_view:
        cargs.append("-bayes_view")
    if circos:
        cargs.append("-circos")
    if custom_packages is not None:
        cargs.extend([
            "-custom",
            custom_packages
        ])
    if mirror is not None:
        cargs.extend([
            "-mirror",
            mirror
        ])
    if help_:
        cargs.append("-help")
    ret = VAfniRPackageInstallOutputs(
        root=execution.output_file("."),
        install_log=execution.output_file("R_packages_installed.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VAfniRPackageInstallOutputs",
    "V__AFNI_R_PACKAGE_INSTALL_METADATA",
    "v__afni_r_package_install",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

OXFORD_ASL_METADATA = Metadata(
    id="1225d433fbe151420738b535aab2a7687f83e294.boutiques",
    name="oxford_asl",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class OxfordAslOutputs(typing.NamedTuple):
    """
    Output object returned when calling `oxford_asl(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_dir: OutputPathType
    """Directory containing the output files"""


def oxford_asl(
    asl_data: InputPathType,
    output_dir_name: str,
    runner: Runner | None = None,
) -> OxfordAslOutputs:
    """
    Calculate perfusion maps from ASL data.
    
    Author: FMRIB Centre, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/oxford_asl
    
    Args:
        asl_data: Input ASL data.
        output_dir_name: Output directory name.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `OxfordAslOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(OXFORD_ASL_METADATA)
    cargs = []
    cargs.append("oxford_asl")
    cargs.append("-i")
    cargs.append(execution.input_file(asl_data))
    cargs.append("-o")
    cargs.append(output_dir_name)
    cargs.append("[options]")
    ret = OxfordAslOutputs(
        root=execution.output_file("."),
        output_dir=execution.output_file(output_dir_name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "OXFORD_ASL_METADATA",
    "OxfordAslOutputs",
    "oxford_asl",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__DJUNCT_DWI_SELECTOR_TCSH_METADATA = Metadata(
    id="9b9b139284ac47200b7f4939472d4e18345a05e3.boutiques",
    name="@djunct_dwi_selector.tcsh",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VDjunctDwiSelectorTcshOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__djunct_dwi_selector_tcsh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfile: OutputPathType
    """The main output file"""


def v__djunct_dwi_selector_tcsh(
    dwi: InputPathType,
    png: InputPathType,
    outfile: str,
    runner: Runner | None = None,
) -> VDjunctDwiSelectorTcshOutputs:
    """
    Selects DWI data and creates a representative image.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/@djunct_dwi_selector.tcsh.html
    
    Args:
        dwi: Input DWI file.
        png: Output PNG image.
        outfile: Path to the output file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VDjunctDwiSelectorTcshOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__DJUNCT_DWI_SELECTOR_TCSH_METADATA)
    cargs = []
    cargs.append("/opt/afni/src/../install/@djunct_dwi_selector.tcsh")
    cargs.append(execution.input_file(dwi))
    cargs.append(execution.input_file(png))
    cargs.append(outfile)
    ret = VDjunctDwiSelectorTcshOutputs(
        root=execution.output_file("."),
        outfile=execution.output_file(outfile),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VDjunctDwiSelectorTcshOutputs",
    "V__DJUNCT_DWI_SELECTOR_TCSH_METADATA",
    "v__djunct_dwi_selector_tcsh",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_EXTREMA_METADATA = Metadata(
    id="5ac881fa6c2cea8b9186981d23d77e4ec3bd7b85.boutiques",
    name="3dExtrema",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dExtremaOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_extrema(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_head_file: OutputPathType | None
    """Output HEAD file"""
    output_brik_file: OutputPathType | None
    """Output BRIK file"""


def v_3d_extrema(
    input_dataset: InputPathType,
    output_prefix: str | None = None,
    output_session: str | None = None,
    quiet: bool = False,
    mask_file: InputPathType | None = None,
    mask_threshold: float | None = None,
    data_threshold: float | None = None,
    n_best: float | None = None,
    separation_distance: float | None = None,
    minima: bool = False,
    maxima: bool = False,
    strict: bool = False,
    partial: bool = False,
    interior: bool = False,
    closure: bool = False,
    slice_: bool = False,
    volume: bool = False,
    remove: bool = False,
    average: bool = False,
    weight: bool = False,
    runner: Runner | None = None,
) -> V3dExtremaOutputs:
    """
    Find local extrema (minima or maxima) in 3D datasets.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dExtrema.html
    
    Args:
        input_dataset: Input dataset (e.g. dataset+tlrc'[sub-brick]').
        output_prefix: Prefix for the output dataset name.
        output_session: Directory for the output dataset session.
        quiet: Suppress screen output.
        mask_file: Mask statistic from file.
        mask_threshold: Only voxels whose mask statistic is >= m in absolute\
            value will be considered.
        data_threshold: Only voxels whose value (intensity) is greater than d\
            in absolute value will be considered.
        n_best: Only print the first N extrema.
        separation_distance: Minimum separation distance (in mm) for distinct\
            extrema.
        minima: Find local minima.
        maxima: Find local maxima (default).
        strict: Use strict inequality for extrema (default).
        partial: Use partial inequality for extrema.
        interior: Extrema must be interior points (default).
        closure: Extrema may be boundary points.
        slice_: Consider each slice separately (default).
        volume: Consider the volume as a whole.
        remove: Remove all but strongest of neighboring extrema (default).
        average: Replace neighboring extrema by average.
        weight: Replace neighboring extrema by weighted average.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dExtremaOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_EXTREMA_METADATA)
    cargs = []
    cargs.append("3dExtrema")
    cargs.append(execution.input_file(input_dataset))
    if output_prefix is not None:
        cargs.extend([
            "-prefix",
            output_prefix
        ])
    if output_session is not None:
        cargs.extend([
            "-session",
            output_session
        ])
    if quiet:
        cargs.append("-quiet")
    if mask_file is not None:
        cargs.extend([
            "-mask_file",
            execution.input_file(mask_file)
        ])
    if mask_threshold is not None:
        cargs.extend([
            "-mask_thr",
            str(mask_threshold)
        ])
    if data_threshold is not None:
        cargs.extend([
            "-data_thr",
            str(data_threshold)
        ])
    if n_best is not None:
        cargs.extend([
            "-nbest",
            str(n_best)
        ])
    if separation_distance is not None:
        cargs.extend([
            "-sep_dist",
            str(separation_distance)
        ])
    if minima:
        cargs.append("-minima")
    if maxima:
        cargs.append("-maxima")
    if strict:
        cargs.append("-strict")
    if partial:
        cargs.append("-partial")
    if interior:
        cargs.append("-interior")
    if closure:
        cargs.append("-closure")
    if slice_:
        cargs.append("-slice")
    if volume:
        cargs.append("-volume")
    if remove:
        cargs.append("-remove")
    if average:
        cargs.append("-average")
    if weight:
        cargs.append("-weight")
    ret = V3dExtremaOutputs(
        root=execution.output_file("."),
        output_head_file=execution.output_file(output_prefix + ".HEAD") if (output_prefix is not None) else None,
        output_brik_file=execution.output_file(output_prefix + ".BRIK") if (output_prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dExtremaOutputs",
    "V_3D_EXTREMA_METADATA",
    "v_3d_extrema",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRINFO_METADATA = Metadata(
    id="1a85852eaad4b28e7ec279a82e8e478eaa7037b1.boutiques",
    name="mrinfo",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


@dataclasses.dataclass
class MrinfoProperty:
    """
    any text properties embedded in the image header under the specified key
    (use 'all' to list all keys found).
    """
    key: str
    """any text properties embedded in the image header under the specified key
    (use 'all' to list all keys found)"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-property")
        cargs.append(self.key)
        return cargs


@dataclasses.dataclass
class MrinfoFslgrad:
    """
    Provide the diffusion-weighted gradient scheme used in the acquisition in
    FSL bvecs/bvals format files. If a diffusion gradient scheme is present in
    the input image header, the data provided with this option will be instead
    used.
    """
    bvecs: InputPathType
    """Provide the diffusion-weighted gradient scheme used in the acquisition in
    FSL bvecs/bvals format files. If a diffusion gradient scheme is present in
    the input image header, the data provided with this option will be instead
    used."""
    bvals: InputPathType
    """Provide the diffusion-weighted gradient scheme used in the acquisition in
    FSL bvecs/bvals format files. If a diffusion gradient scheme is present in
    the input image header, the data provided with this option will be instead
    used."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-fslgrad")
        cargs.append(execution.input_file(self.bvecs))
        cargs.append(execution.input_file(self.bvals))
        return cargs


class MrinfoExportGradFslOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrinfoExportGradFsl | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    bvecs_path: OutputPathType
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""
    bvals_path: OutputPathType
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""


@dataclasses.dataclass
class MrinfoExportGradFsl:
    """
    export the diffusion-weighted gradient table to files in FSL (bvecs / bvals)
    format.
    """
    bvecs_path: str
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""
    bvals_path: str
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-export_grad_fsl")
        cargs.append(self.bvecs_path)
        cargs.append(self.bvals_path)
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> MrinfoExportGradFslOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `MrinfoExportGradFslOutputs`).
        """
        ret = MrinfoExportGradFslOutputs(
            root=execution.output_file("."),
            bvecs_path=execution.output_file(self.bvecs_path),
            bvals_path=execution.output_file(self.bvals_path),
        )
        return ret


class MrinfoExportPeEddyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrinfoExportPeEddy | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    config: OutputPathType
    """export phase-encoding information to an EDDY-style config / index file
    pair"""
    indices: OutputPathType
    """export phase-encoding information to an EDDY-style config / index file
    pair"""


@dataclasses.dataclass
class MrinfoExportPeEddy:
    """
    export phase-encoding information to an EDDY-style config / index file pair.
    """
    config: str
    """export phase-encoding information to an EDDY-style config / index file
    pair"""
    indices: str
    """export phase-encoding information to an EDDY-style config / index file
    pair"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-export_pe_eddy")
        cargs.append(self.config)
        cargs.append(self.indices)
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> MrinfoExportPeEddyOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `MrinfoExportPeEddyOutputs`).
        """
        ret = MrinfoExportPeEddyOutputs(
            root=execution.output_file("."),
            config=execution.output_file(self.config),
            indices=execution.output_file(self.indices),
        )
        return ret


@dataclasses.dataclass
class MrinfoConfig:
    """
    temporarily set the value of an MRtrix config file entry.
    """
    key: str
    """temporarily set the value of an MRtrix config file entry."""
    value: str
    """temporarily set the value of an MRtrix config file entry."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-config")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


class MrinfoOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mrinfo(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    json_keyval: OutputPathType | None
    """export header key/value entries to a JSON file """
    json_all: OutputPathType | None
    """export all header contents to a JSON file """
    export_grad_mrtrix: OutputPathType | None
    """export the diffusion-weighted gradient table to file in MRtrix format """
    export_pe_table: OutputPathType | None
    """export phase-encoding table to file """
    export_grad_fsl: MrinfoExportGradFslOutputs | None
    """Outputs from `MrinfoExportGradFsl`."""
    export_pe_eddy: MrinfoExportPeEddyOutputs | None
    """Outputs from `MrinfoExportPeEddy`."""


def mrinfo(
    image: list[InputPathType],
    all_: bool = False,
    name: bool = False,
    format_: bool = False,
    ndim: bool = False,
    size: bool = False,
    spacing: bool = False,
    datatype: bool = False,
    strides: bool = False,
    offset: bool = False,
    multiplier: bool = False,
    transform: bool = False,
    property_: list[MrinfoProperty] | None = None,
    json_keyval: str | None = None,
    json_all: str | None = None,
    grad: InputPathType | None = None,
    fslgrad: MrinfoFslgrad | None = None,
    bvalue_scaling: str | None = None,
    export_grad_mrtrix: str | None = None,
    export_grad_fsl: MrinfoExportGradFsl | None = None,
    dwgrad: bool = False,
    shell_bvalues: bool = False,
    shell_sizes: bool = False,
    shell_indices: bool = False,
    export_pe_table: str | None = None,
    export_pe_eddy: MrinfoExportPeEddy | None = None,
    petable: bool = False,
    nodelete: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrinfoConfig] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrinfoOutputs:
    """
    Display image header information, or extract specific information from the
    header.
    
    By default, all information contained in each image header will be printed
    to the console in a reader-friendly format.
    
    Alternatively, command-line options may be used to extract specific details
    from the header(s); these are printed to the console in a format more
    appropriate for scripting purposes or piping to file. If multiple options
    and/or images are provided, the requested header fields will be printed in
    the order in which they appear in the help page, with all requested details
    from each input image in sequence printed before the next image is
    processed.
    
    The command can also write the diffusion gradient table from a single input
    image to file; either in the MRtrix or FSL format (bvecs/bvals file pair;
    includes appropriate diffusion gradient vector reorientation)
    
    The -dwgrad, -export_* and -shell_* options provide (information about) the
    diffusion weighting gradient table after it has been processed by the
    MRtrix3 back-end (vectors normalised, b-values scaled by the square of the
    vector norm, depending on the -bvalue_scaling option). To see the raw
    gradient table information as stored in the image header, i.e. without
    MRtrix3 back-end processing, use "-property dw_scheme".
    
    The -bvalue_scaling option controls an aspect of the import of diffusion
    gradient tables. When the input diffusion-weighting direction vectors have
    norms that differ substantially from unity, the b-values will be scaled by
    the square of their corresponding vector norm (this is how multi-shell
    acquisitions are frequently achieved on scanner platforms). However in some
    rare instances, the b-values may be correct, despite the vectors not being
    of unit norm (or conversely, the b-values may need to be rescaled even
    though the vectors are close to unit norm). This option allows the user to
    control this operation and override MRrtix3's automatic detection.
    
    References:
    
    .
    
    Author: J-Donald Tournier (jdtournier@gmail.com) and Robert E. Smith
    (robert.smith@florey.edu.au)
    
    URL: https://mrtrix.readthedocs.io/en/latest/reference/commands/mrinfo.html
    
    Args:
        image: the input image(s).
        all_: print all properties, rather than the first and last 2 of each.
        name: print the file system path of the image.
        format_: image file format.
        ndim: number of image dimensions.
        size: image size along each axis.
        spacing: voxel spacing along each image dimension.
        datatype: data type used for image data storage.
        strides: data strides i.e. order and direction of axes data layout.
        offset: image intensity offset.
        multiplier: image intensity multiplier.
        transform: the transformation from image coordinates [mm] to scanner /\
            real world coordinates [mm].
        property_: any text properties embedded in the image header under the\
            specified key (use 'all' to list all keys found).
        json_keyval: export header key/value entries to a JSON file.
        json_all: export all header contents to a JSON file.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvalue_scaling: enable or disable scaling of diffusion b-values by the\
            square of the corresponding DW gradient norm (see Desciption). Valid\
            choices are yes/no, true/false, 0/1 (default: automatic).
        export_grad_mrtrix: export the diffusion-weighted gradient table to\
            file in MRtrix format.
        export_grad_fsl: export the diffusion-weighted gradient table to files\
            in FSL (bvecs / bvals) format.
        dwgrad: the diffusion-weighting gradient table, as interpreted by\
            MRtrix3.
        shell_bvalues: list the average b-value of each shell.
        shell_sizes: list the number of volumes in each shell.
        shell_indices: list the image volumes attributed to each b-value shell.
        export_pe_table: export phase-encoding table to file.
        export_pe_eddy: export phase-encoding information to an EDDY-style\
            config / index file pair.
        petable: print the phase encoding table.
        nodelete: don't delete temporary images or images passed to mrinfo via\
            Unix pipes.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrinfoOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRINFO_METADATA)
    cargs = []
    cargs.append("mrinfo")
    if all_:
        cargs.append("-all")
    if name:
        cargs.append("-name")
    if format_:
        cargs.append("-format")
    if ndim:
        cargs.append("-ndim")
    if size:
        cargs.append("-size")
    if spacing:
        cargs.append("-spacing")
    if datatype:
        cargs.append("-datatype")
    if strides:
        cargs.append("-strides")
    if offset:
        cargs.append("-offset")
    if multiplier:
        cargs.append("-multiplier")
    if transform:
        cargs.append("-transform")
    if property_ is not None:
        cargs.extend([a for c in [s.run(execution) for s in property_] for a in c])
    if json_keyval is not None:
        cargs.extend([
            "-json_keyval",
            json_keyval
        ])
    if json_all is not None:
        cargs.extend([
            "-json_all",
            json_all
        ])
    if grad is not None:
        cargs.extend([
            "-grad",
            execution.input_file(grad)
        ])
    if fslgrad is not None:
        cargs.extend(fslgrad.run(execution))
    if bvalue_scaling is not None:
        cargs.extend([
            "-bvalue_scaling",
            bvalue_scaling
        ])
    if export_grad_mrtrix is not None:
        cargs.extend([
            "-export_grad_mrtrix",
            export_grad_mrtrix
        ])
    if export_grad_fsl is not None:
        cargs.extend(export_grad_fsl.run(execution))
    if dwgrad:
        cargs.append("-dwgrad")
    if shell_bvalues:
        cargs.append("-shell_bvalues")
    if shell_sizes:
        cargs.append("-shell_sizes")
    if shell_indices:
        cargs.append("-shell_indices")
    if export_pe_table is not None:
        cargs.extend([
            "-export_pe_table",
            export_pe_table
        ])
    if export_pe_eddy is not None:
        cargs.extend(export_pe_eddy.run(execution))
    if petable:
        cargs.append("-petable")
    if nodelete:
        cargs.append("-nodelete")
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([a for c in [s.run(execution) for s in config] for a in c])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    cargs.extend([execution.input_file(f) for f in image])
    ret = MrinfoOutputs(
        root=execution.output_file("."),
        json_keyval=execution.output_file(json_keyval) if (json_keyval is not None) else None,
        json_all=execution.output_file(json_all) if (json_all is not None) else None,
        export_grad_mrtrix=execution.output_file(export_grad_mrtrix) if (export_grad_mrtrix is not None) else None,
        export_pe_table=execution.output_file(export_pe_table) if (export_pe_table is not None) else None,
        export_grad_fsl=export_grad_fsl.outputs(execution) if export_grad_fsl else None,
        export_pe_eddy=export_pe_eddy.outputs(execution) if export_pe_eddy else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRINFO_METADATA",
    "MrinfoConfig",
    "MrinfoExportGradFsl",
    "MrinfoExportGradFslOutputs",
    "MrinfoExportPeEddy",
    "MrinfoExportPeEddyOutputs",
    "MrinfoFslgrad",
    "MrinfoOutputs",
    "MrinfoProperty",
    "mrinfo",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CHECK_FEAT_METADATA = Metadata(
    id="88315ec54afacdeb331bb12bda2509e0741bd892.boutiques",
    name="checkFEAT",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class CheckFeatOutputs(typing.NamedTuple):
    """
    Output object returned when calling `check_feat(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_report: OutputPathType
    """Output HTML report"""
    output_report_log: OutputPathType
    """Output HTML report log"""


def check_feat(
    report_file: InputPathType,
    report_log_file: InputPathType,
    runner: Runner | None = None,
) -> CheckFeatOutputs:
    """
    Perform checks on FEAT analysis results.
    
    Author: YourName
    
    URL: http://example.com/checkFEAT
    
    Args:
        report_file: Path to the HTML report.
        report_log_file: Path to the HTML report log.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CheckFeatOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CHECK_FEAT_METADATA)
    cargs = []
    cargs.append("checkFEAT")
    cargs.append(execution.input_file(report_file))
    cargs.append(execution.input_file(report_log_file))
    ret = CheckFeatOutputs(
        root=execution.output_file("."),
        output_report=execution.output_file("output_report.html"),
        output_report_log=execution.output_file("output_report_log.html"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CHECK_FEAT_METADATA",
    "CheckFeatOutputs",
    "check_feat",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CLUSTER2HTML_METADATA = Metadata(
    id="b70bb6520b5d33f9d586ec00acad339b7b15f08c.boutiques",
    name="cluster2html",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class Cluster2htmlOutputs(typing.NamedTuple):
    """
    Output object returned when calling `cluster2html(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def cluster2html(
    featdir: str,
    inroot: str,
    std_flag: bool = False,
    runner: Runner | None = None,
) -> Cluster2htmlOutputs:
    """
    Generates an HTML report from cluster-based FEAT analysis.
    
    Author: Oxford Centre for Functional MRI of the Brain (FMRIB)
    
    Args:
        featdir: Directory containing the FEAT analysis results.
        inroot: Root name for cluster files (should not contain the _std\
            extension).
        std_flag: Indicate that the input files contain the _std extension.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Cluster2htmlOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CLUSTER2HTML_METADATA)
    cargs = []
    cargs.append("cluster2html")
    cargs.append(featdir)
    cargs.append(inroot)
    if std_flag:
        cargs.append("-std")
    ret = Cluster2htmlOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CLUSTER2HTML_METADATA",
    "Cluster2htmlOutputs",
    "cluster2html",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FSL_MRS_METADATA = Metadata(
    id="00847a495adc2e0e69dfce6d1d732fdf079b343b.boutiques",
    name="fsl_mrs",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class FslMrsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fsl_mrs(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def fsl_mrs(
    data: InputPathType,
    basis: InputPathType,
    output: str,
    algo: str | None = None,
    ignore_metab: list[str] | None = None,
    keep_metab: list[str] | None = None,
    combine_metab: list[str] | None = None,
    ppmlim: list[float] | None = None,
    h2o_file: InputPathType | None = None,
    baseline_order: float | None = None,
    metab_groups: list[str] | None = None,
    add_mm_flag: bool = False,
    add_mm_mega_flag: bool = False,
    lorentzian_flag: bool = False,
    ind_scaling: list[str] | None = None,
    disable_mh_priors_flag: bool = False,
    t1_image: InputPathType | None = None,
    te: float | None = None,
    tr: float | None = None,
    tissue_frac: list[str] | None = None,
    internal_ref: list[str] | None = None,
    wref_metabolite: str | None = None,
    ref_protons: float | None = None,
    ref_int_limits: list[float] | None = None,
    h2o_scale: float | None = None,
    report_flag: bool = False,
    verbose_flag: bool = False,
    overwrite_flag: bool = False,
    conj_fid: bool = False,
    no_conj_fid: bool = False,
    conj_basis: bool = False,
    no_conj_basis: bool = False,
    no_rescale: bool = False,
    config: InputPathType | None = None,
    runner: Runner | None = None,
) -> FslMrsOutputs:
    """
    FSL Magnetic Resonance Spectroscopy Wrapper Script.
    
    Author: FMRIB Centre, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/
    
    Args:
        data: Input FID file.
        basis: .BASIS file or folder containing basis spectra (will read all\
            files within).
        output: Output folder.
        algo: Algorithm [Newton (fast, default) or MH (slow)].
        ignore_metab: Ignore certain metabolites [repeatable].
        keep_metab: Only keep these metabolites.
        combine_metab: Combine certain metabolites [repeatable].
        ppmlim: Limit the fit to a frequency range (default=(.2,4.2)).
        h2o_file: Input .H2O file for quantification.
        baseline_order: Order of baseline polynomial (default=2, -1 disables).
        metab_groups: Metabolite groups: list of groups or list of names for\
            independent groups.
        add_mm_flag: Include default macromolecule peaks.
        add_mm_mega_flag: Include default MEGA-PRESS macromolecule peaks. This\
            option is experimental!.
        lorentzian_flag: Enable purely Lorentzian broadening (default is Voigt).
        ind_scaling: List of basis spectra to scale independently of other\
            basis spectra.
        disable_mh_priors_flag: Disable MH priors.
        t1_image: Structural image (for report).
        te: Echo time for relaxation correction (ms).
        tr: Repetition time for relaxation correction (s).
        tissue_frac: Fractional tissue volumes for WM, GM, CSF or json\
            segmentation file. Defaults to pure water scaling.
        internal_ref: Metabolite(s) used as an internal reference. Defaults to\
            tCr (Cr+PCr).
        wref_metabolite: Metabolite(s) used as the reference for water scaling.\
            Uses internal defaults otherwise.
        ref_protons: Number of protons that reference metabolite is equivalent\
            to. No effect without setting --wref_metabolite.
        ref_int_limits: Reference spectrum integration limits (low, high). No\
            effect without setting --wref_metabolite.
        h2o_scale: Additional scaling modifier for external water referencing.
        report_flag: Output HTML report.
        verbose_flag: Spit out verbose info.
        overwrite_flag: Overwrite existing output folder.
        conj_fid: Force conjugation of FID.
        no_conj_fid: Forbid automatic conjugation of FID.
        conj_basis: Force conjugation of basis.
        no_conj_basis: Forbid automatic conjugation of basis.
        no_rescale: Forbid rescaling of FID/basis/H2O.
        config: Configuration file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FslMrsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FSL_MRS_METADATA)
    cargs = []
    cargs.append("fsl_mrs")
    cargs.append(execution.input_file(data))
    cargs.append(execution.input_file(basis))
    cargs.append(output)
    if algo is not None:
        cargs.extend([
            "--algo",
            algo
        ])
    if ignore_metab is not None:
        cargs.extend([
            "--ignore",
            *ignore_metab
        ])
    if keep_metab is not None:
        cargs.extend([
            "--keep",
            *keep_metab
        ])
    if combine_metab is not None:
        cargs.extend([
            "--combine",
            *combine_metab
        ])
    if ppmlim is not None:
        cargs.extend([
            "--ppmlim",
            *map(str, ppmlim)
        ])
    if h2o_file is not None:
        cargs.extend([
            "--h2o",
            execution.input_file(h2o_file)
        ])
    if baseline_order is not None:
        cargs.extend([
            "--baseline_order",
            str(baseline_order)
        ])
    if metab_groups is not None:
        cargs.extend([
            "--metab_groups",
            *metab_groups
        ])
    if add_mm_flag:
        cargs.append("--add_MM")
    if add_mm_mega_flag:
        cargs.append("--add_MM_MEGA")
    if lorentzian_flag:
        cargs.append("--lorentzian")
    if ind_scaling is not None:
        cargs.extend([
            "--ind_scale",
            *ind_scaling
        ])
    if disable_mh_priors_flag:
        cargs.append("--disable_MH_priors")
    if t1_image is not None:
        cargs.extend([
            "--t1",
            execution.input_file(t1_image)
        ])
    if te is not None:
        cargs.extend([
            "--TE",
            str(te)
        ])
    if tr is not None:
        cargs.extend([
            "--TR",
            str(tr)
        ])
    if tissue_frac is not None:
        cargs.extend([
            "--tissue_frac",
            *tissue_frac
        ])
    if internal_ref is not None:
        cargs.extend([
            "--internal_ref",
            *internal_ref
        ])
    if wref_metabolite is not None:
        cargs.extend([
            "--wref_metabolite",
            wref_metabolite
        ])
    if ref_protons is not None:
        cargs.extend([
            "--ref_protons",
            str(ref_protons)
        ])
    if ref_int_limits is not None:
        cargs.extend([
            "--ref_int_limits",
            *map(str, ref_int_limits)
        ])
    if h2o_scale is not None:
        cargs.extend([
            "--h2o_scale",
            str(h2o_scale)
        ])
    if report_flag:
        cargs.append("--report")
    if verbose_flag:
        cargs.append("--verbose")
    if overwrite_flag:
        cargs.append("--overwrite")
    if conj_fid:
        cargs.append("--conj_fid")
    if no_conj_fid:
        cargs.append("--no_conj_fid")
    if conj_basis:
        cargs.append("--conj_basis")
    if no_conj_basis:
        cargs.append("--no_conj_basis")
    if no_rescale:
        cargs.append("--no_rescale")
    if config is not None:
        cargs.extend([
            "--config",
            execution.input_file(config)
        ])
    ret = FslMrsOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FSL_MRS_METADATA",
    "FslMrsOutputs",
    "fsl_mrs",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CIFTI_CONVERT_METADATA = Metadata(
    id="f620818fd4e7449f16b3a3864a16b31df547c1e9.boutiques",
    name="cifti-convert",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class CiftiConvertToGiftiExt:
    """
    convert to GIFTI external binary.
    """
    cifti_in: InputPathType
    """the input cifti file"""
    gifti_out: str
    """output - the output gifti file"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-to-gifti-ext")
        cargs.append(execution.input_file(self.cifti_in))
        cargs.append(self.gifti_out)
        return cargs


@dataclasses.dataclass
class CiftiConvertResetTimepoints:
    """
    reset the mapping along rows to timepoints, taking length from the gifti
    file.
    """
    timestep: float
    """the desired time between frames"""
    timestart: float
    """the desired time offset of the initial frame"""
    opt_unit_unit: str | None = None
    """use a unit other than time: unit identifier (default SECOND)"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-reset-timepoints")
        cargs.append(str(self.timestep))
        cargs.append(str(self.timestart))
        if self.opt_unit_unit is not None:
            cargs.extend([
                "-unit",
                self.opt_unit_unit
            ])
        return cargs


@dataclasses.dataclass
class CiftiConvertReplaceBinary:
    """
    replace data with a binary file.
    """
    binary_in: str
    """the binary file that contains replacement data"""
    opt_flip_endian: bool = False
    """byteswap the binary file"""
    opt_transpose: bool = False
    """transpose the binary file"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-replace-binary")
        cargs.append(self.binary_in)
        if self.opt_flip_endian:
            cargs.append("-flip-endian")
        if self.opt_transpose:
            cargs.append("-transpose")
        return cargs


class CiftiConvertFromGiftiExtOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiConvertFromGiftiExt | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


@dataclasses.dataclass
class CiftiConvertFromGiftiExt:
    """
    convert a GIFTI made with this command back into a CIFTI.
    """
    gifti_in: str
    """the input gifti file"""
    cifti_out: str
    """the output cifti file"""
    reset_timepoints: CiftiConvertResetTimepoints | None = None
    """reset the mapping along rows to timepoints, taking length from the gifti
    file"""
    opt_reset_scalars: bool = False
    """reset mapping along rows to scalars, taking length from the gifti file"""
    opt_column_reset_scalars: bool = False
    """reset mapping along columns to scalar (useful for changing number of
    series in a sdseries file)"""
    replace_binary: CiftiConvertReplaceBinary | None = None
    """replace data with a binary file"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-from-gifti-ext")
        cargs.append(self.gifti_in)
        cargs.append(self.cifti_out)
        if self.reset_timepoints is not None:
            cargs.extend(self.reset_timepoints.run(execution))
        if self.opt_reset_scalars:
            cargs.append("-reset-scalars")
        if self.opt_column_reset_scalars:
            cargs.append("-column-reset-scalars")
        if self.replace_binary is not None:
            cargs.extend(self.replace_binary.run(execution))
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> CiftiConvertFromGiftiExtOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `CiftiConvertFromGiftiExtOutputs`).
        """
        ret = CiftiConvertFromGiftiExtOutputs(
            root=execution.output_file("."),
            cifti_out=execution.output_file(self.cifti_out),
        )
        return ret


class CiftiConvertToNiftiOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiConvertToNifti | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    nifti_out: OutputPathType
    """the output nifti file"""


@dataclasses.dataclass
class CiftiConvertToNifti:
    """
    convert to NIFTI1.
    """
    cifti_in: InputPathType
    """the input cifti file"""
    nifti_out: str
    """the output nifti file"""
    opt_smaller_file: bool = False
    """use better-fitting dimension lengths"""
    opt_smaller_dims: bool = False
    """minimize the largest dimension, for tools that don't like large
    indices"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-to-nifti")
        cargs.append(execution.input_file(self.cifti_in))
        cargs.append(self.nifti_out)
        if self.opt_smaller_file:
            cargs.append("-smaller-file")
        if self.opt_smaller_dims:
            cargs.append("-smaller-dims")
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> CiftiConvertToNiftiOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `CiftiConvertToNiftiOutputs`).
        """
        ret = CiftiConvertToNiftiOutputs(
            root=execution.output_file("."),
            nifti_out=execution.output_file(self.nifti_out),
        )
        return ret


@dataclasses.dataclass
class CiftiConvertResetTimepoints_:
    """
    reset the mapping along rows to timepoints, taking length from the nifti
    file.
    """
    timestep: float
    """the desired time between frames"""
    timestart: float
    """the desired time offset of the initial frame"""
    opt_unit_unit: str | None = None
    """use a unit other than time: unit identifier (default SECOND)"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-reset-timepoints")
        cargs.append(str(self.timestep))
        cargs.append(str(self.timestart))
        if self.opt_unit_unit is not None:
            cargs.extend([
                "-unit",
                self.opt_unit_unit
            ])
        return cargs


class CiftiConvertFromNiftiOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiConvertFromNifti | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


@dataclasses.dataclass
class CiftiConvertFromNifti:
    """
    convert a NIFTI (1 or 2) file made with this command back into CIFTI.
    """
    nifti_in: InputPathType
    """the input nifti file"""
    cifti_template: InputPathType
    """a cifti file with the dimension(s) and mapping(s) that should be used"""
    cifti_out: str
    """the output cifti file"""
    reset_timepoints: CiftiConvertResetTimepoints_ | None = None
    """reset the mapping along rows to timepoints, taking length from the nifti
    file"""
    opt_reset_scalars: bool = False
    """reset mapping along rows to scalars, taking length from the nifti file"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-from-nifti")
        cargs.append(execution.input_file(self.nifti_in))
        cargs.append(execution.input_file(self.cifti_template))
        cargs.append(self.cifti_out)
        if self.reset_timepoints is not None:
            cargs.extend(self.reset_timepoints.run(execution))
        if self.opt_reset_scalars:
            cargs.append("-reset-scalars")
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> CiftiConvertFromNiftiOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `CiftiConvertFromNiftiOutputs`).
        """
        ret = CiftiConvertFromNiftiOutputs(
            root=execution.output_file("."),
            cifti_out=execution.output_file(self.cifti_out),
        )
        return ret


@dataclasses.dataclass
class CiftiConvertToText:
    """
    convert to a plain text file.
    """
    cifti_in: InputPathType
    """the input cifti file"""
    text_out: str
    """output - the output text file"""
    opt_col_delim_delim_string: str | None = None
    """choose string to put between elements in a row: the string to use
    (default is a tab character)"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-to-text")
        cargs.append(execution.input_file(self.cifti_in))
        cargs.append(self.text_out)
        if self.opt_col_delim_delim_string is not None:
            cargs.extend([
                "-col-delim",
                self.opt_col_delim_delim_string
            ])
        return cargs


@dataclasses.dataclass
class CiftiConvertResetTimepoints_2:
    """
    reset the mapping along rows to timepoints, taking length from the text
    file.
    """
    timestep: float
    """the desired time between frames"""
    timestart: float
    """the desired time offset of the initial frame"""
    opt_unit_unit: str | None = None
    """use a unit other than time: unit identifier (default SECOND)"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-reset-timepoints")
        cargs.append(str(self.timestep))
        cargs.append(str(self.timestart))
        if self.opt_unit_unit is not None:
            cargs.extend([
                "-unit",
                self.opt_unit_unit
            ])
        return cargs


class CiftiConvertFromTextOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiConvertFromText | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


@dataclasses.dataclass
class CiftiConvertFromText:
    """
    convert from plain text to cifti.
    """
    text_in: str
    """the input text file"""
    cifti_template: InputPathType
    """a cifti file with the dimension(s) and mapping(s) that should be used"""
    cifti_out: str
    """the output cifti file"""
    opt_col_delim_delim_string: str | None = None
    """specify string that is between elements in a row: the string to use
    (default is any whitespace)"""
    reset_timepoints: CiftiConvertResetTimepoints_2 | None = None
    """reset the mapping along rows to timepoints, taking length from the text
    file"""
    opt_reset_scalars: bool = False
    """reset mapping along rows to scalars, taking length from the text file"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-from-text")
        cargs.append(self.text_in)
        cargs.append(execution.input_file(self.cifti_template))
        cargs.append(self.cifti_out)
        if self.opt_col_delim_delim_string is not None:
            cargs.extend([
                "-col-delim",
                self.opt_col_delim_delim_string
            ])
        if self.reset_timepoints is not None:
            cargs.extend(self.reset_timepoints.run(execution))
        if self.opt_reset_scalars:
            cargs.append("-reset-scalars")
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> CiftiConvertFromTextOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `CiftiConvertFromTextOutputs`).
        """
        ret = CiftiConvertFromTextOutputs(
            root=execution.output_file("."),
            cifti_out=execution.output_file(self.cifti_out),
        )
        return ret


class CiftiConvertOutputs(typing.NamedTuple):
    """
    Output object returned when calling `cifti_convert(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    from_gifti_ext: CiftiConvertFromGiftiExtOutputs | None
    """Outputs from `CiftiConvertFromGiftiExt`."""
    to_nifti: CiftiConvertToNiftiOutputs | None
    """Outputs from `CiftiConvertToNifti`."""
    from_nifti: CiftiConvertFromNiftiOutputs | None
    """Outputs from `CiftiConvertFromNifti`."""
    from_text: CiftiConvertFromTextOutputs | None
    """Outputs from `CiftiConvertFromText`."""


def cifti_convert(
    to_gifti_ext: CiftiConvertToGiftiExt | None = None,
    from_gifti_ext: CiftiConvertFromGiftiExt | None = None,
    to_nifti: CiftiConvertToNifti | None = None,
    from_nifti: CiftiConvertFromNifti | None = None,
    to_text: CiftiConvertToText | None = None,
    from_text: CiftiConvertFromText | None = None,
    runner: Runner | None = None,
) -> CiftiConvertOutputs:
    """
    Dump cifti matrix into other formats.
    
    This command is used to convert a full CIFTI matrix to/from formats that can
    be used by programs that don't understand CIFTI. You must specify exactly
    one of -to-gifti-ext, -from-gifti-ext, -to-nifti, -from-nifti, -to-text, or
    -from-text.
    
    If you want to write an existing CIFTI file with a different CIFTI version,
    see -file-convert, and its -cifti-version-convert option.
    
    If you want part of the CIFTI file as a metric, label, or volume file, see
    -cifti-separate. If you want to create a CIFTI file from metric and/or
    volume files, see the -cifti-create-* commands.
    
    If you want to import a matrix that is restricted to an ROI, first create a
    template CIFTI file matching that ROI using a -cifti-create-* command. After
    importing to CIFTI, you can then expand the file into a standard
    brainordinates space with -cifti-create-dense-from-template. If you want to
    export only part of a CIFTI file, first create an roi-restricted CIFTI file
    with -cifti-restrict-dense-mapping.
    
    The -transpose option to -from-gifti-ext is needed if the replacement binary
    file is in column-major order.
    
    The -unit options accept these values:
    
    SECOND
    HERTZ
    METER
    RADIAN.
    
    Author: Washington University School of Medicin
    
    Args:
        to_gifti_ext: convert to GIFTI external binary.
        from_gifti_ext: convert a GIFTI made with this command back into a\
            CIFTI.
        to_nifti: convert to NIFTI1.
        from_nifti: convert a NIFTI (1 or 2) file made with this command back\
            into CIFTI.
        to_text: convert to a plain text file.
        from_text: convert from plain text to cifti.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiConvertOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_CONVERT_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-convert")
    if to_gifti_ext is not None:
        cargs.extend(to_gifti_ext.run(execution))
    if from_gifti_ext is not None:
        cargs.extend(from_gifti_ext.run(execution))
    if to_nifti is not None:
        cargs.extend(to_nifti.run(execution))
    if from_nifti is not None:
        cargs.extend(from_nifti.run(execution))
    if to_text is not None:
        cargs.extend(to_text.run(execution))
    if from_text is not None:
        cargs.extend(from_text.run(execution))
    ret = CiftiConvertOutputs(
        root=execution.output_file("."),
        from_gifti_ext=from_gifti_ext.outputs(execution) if from_gifti_ext else None,
        to_nifti=to_nifti.outputs(execution) if to_nifti else None,
        from_nifti=from_nifti.outputs(execution) if from_nifti else None,
        from_text=from_text.outputs(execution) if from_text else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CIFTI_CONVERT_METADATA",
    "CiftiConvertFromGiftiExt",
    "CiftiConvertFromGiftiExtOutputs",
    "CiftiConvertFromNifti",
    "CiftiConvertFromNiftiOutputs",
    "CiftiConvertFromText",
    "CiftiConvertFromTextOutputs",
    "CiftiConvertOutputs",
    "CiftiConvertReplaceBinary",
    "CiftiConvertResetTimepoints",
    "CiftiConvertResetTimepoints_",
    "CiftiConvertResetTimepoints_2",
    "CiftiConvertToGiftiExt",
    "CiftiConvertToNifti",
    "CiftiConvertToNiftiOutputs",
    "CiftiConvertToText",
    "cifti_convert",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FILE_TOOL_METADATA = Metadata(
    id="cb8493bc920691c7d01b60c7f97dde0b5301728c.boutiques",
    name="file_tool",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class FileToolOutputs(typing.NamedTuple):
    """
    Output object returned when calling `file_tool(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    modified_file: OutputPathType
    """The modified output file"""


def file_tool(
    runner: Runner | None = None,
) -> FileToolOutputs:
    """
    Program to display or modify sections of a file.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/file_tool.html
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FileToolOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FILE_TOOL_METADATA)
    cargs = []
    cargs.append("file_tool")
    cargs.append("[OPTIONS]")
    cargs.append("-infiles")
    cargs.append("[INPUT_FILES...]")
    ret = FileToolOutputs(
        root=execution.output_file("."),
        modified_file=execution.output_file("[PREFIX]"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FILE_TOOL_METADATA",
    "FileToolOutputs",
    "file_tool",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__STATAUXCODE_METADATA = Metadata(
    id="6684228b34c86f8889eec8a941e2b73507ea349a.boutiques",
    name="@statauxcode",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VStatauxcodeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__statauxcode(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """Output file containing the result of the conversion"""


def v__statauxcode(
    code_: str,
    runner: Runner | None = None,
) -> VStatauxcodeOutputs:
    """
    Returns the name or number of a statistics code based on specified mappings.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/@statauxcode.html
    
    Args:
        code_: The statistical code or its numerical equivalent to be\
            converted.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VStatauxcodeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__STATAUXCODE_METADATA)
    cargs = []
    cargs.append("@statauxcode")
    cargs.append(code_)
    ret = VStatauxcodeOutputs(
        root=execution.output_file("."),
        output=execution.output_file("output.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VStatauxcodeOutputs",
    "V__STATAUXCODE_METADATA",
    "v__statauxcode",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3DPC_METADATA = Metadata(
    id="2ca991097ca258f9139795385db58ddbb0bb2f3e.boutiques",
    name="3dpc",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dpcOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3dpc(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_dataset: OutputPathType
    """Output dataset file"""
    output_header: OutputPathType
    """Output dataset header file"""
    output_eig: OutputPathType
    """File with computed eigenvalues"""
    output_vec: OutputPathType
    """File with all eigen-timeseries"""
    output_individual_vec: OutputPathType
    """File with individual eigenvalue timeseries"""


def v_3dpc(
    runner: Runner | None = None,
) -> V3dpcOutputs:
    """
    Principal Component Analysis of 3D Datasets.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dpc.html
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dpcOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3DPC_METADATA)
    cargs = []
    cargs.append("3dpc")
    cargs.append("[OPTIONS]")
    cargs.append("[INPUT_DATASETS...]")
    ret = V3dpcOutputs(
        root=execution.output_file("."),
        output_dataset=execution.output_file("[PREFIX]+orig.BRIK"),
        output_header=execution.output_file("[PREFIX]+orig.HEAD"),
        output_eig=execution.output_file("[PREFIX]_eig.1D"),
        output_vec=execution.output_file("[PREFIX]_vec.1D"),
        output_individual_vec=execution.output_file("[PREFIX][NN].1D"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dpcOutputs",
    "V_3DPC_METADATA",
    "v_3dpc",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

PNM_STAGE1_METADATA = Metadata(
    id="dd95e06b08a085e32cb0c931cacdd986c50dea4f.boutiques",
    name="pnm_stage1",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class PnmStage1Outputs(typing.NamedTuple):
    """
    Output object returned when calling `pnm_stage1(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfile: OutputPathType
    """Output physiological data file"""


def pnm_stage1(
    infile: InputPathType,
    out_basename: str,
    runner: Runner | None = None,
) -> PnmStage1Outputs:
    """
    Physiological data preprocessing for FSL.
    
    Author: University of Oxford (Mark Jenkinson)
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/PNM#The_popp_program
    
    Args:
        infile: Input physiological data filename (text format).
        out_basename: Output basename for physiological data and\
            timing/triggers (no extensions).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `PnmStage1Outputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(PNM_STAGE1_METADATA)
    cargs = []
    cargs.append("popp")
    cargs.extend([
        "-i",
        execution.input_file(infile)
    ])
    cargs.extend([
        "-o",
        out_basename
    ])
    cargs.append("[OPTIONAL_ARGUMENTS]")
    ret = PnmStage1Outputs(
        root=execution.output_file("."),
        outfile=execution.output_file(out_basename + "_output.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "PNM_STAGE1_METADATA",
    "PnmStage1Outputs",
    "pnm_stage1",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FAT_PROC_AXIALIZE_ANAT_METADATA = Metadata(
    id="a48c3b651a436a8583174469aa922aea5343ea2f.boutiques",
    name="fat_proc_axialize_anat",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class FatProcAxializeAnatOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fat_proc_axialize_anat(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """An anatomical data set that is regularly situated within its FOV
    volume"""
    working_directory: OutputPathType | None
    """The working directory with intermediate files"""


def fat_proc_axialize_anat(
    in_file: InputPathType,
    ref_file: InputPathType,
    prefix: str,
    mode_t2w: bool = False,
    mode_t1w: bool = False,
    workdir: str | None = None,
    out_match_ref: bool = False,
    do_ceil_out: bool = False,
    extra_al_wtmask: InputPathType | None = None,
    extra_al_cost: str | None = None,
    extra_al_opts: str | None = None,
    focus_mask: InputPathType | None = None,
    focus_by_ss: bool = False,
    remove_inf_sli: float | None = None,
    pre_align_center_mass: bool = False,
    pre_center_mass: bool = False,
    post_lr_symm: bool = False,
    no_pre_lr_symm: bool = False,
    no_clean: bool = False,
    no_qc_view: bool = False,
    qc_prefix: str | None = None,
    runner: Runner | None = None,
) -> FatProcAxializeAnatOutputs:
    """
    Helps align the major axes of an anatomical volume to those of the volumetric
    field of view.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/fat_proc_axialize_anat.html
    
    Args:
        in_file: Input anatomical volume full name.
        ref_file: Reference volume full name, such as TT or MNI.
        prefix: Output prefix for files and snapshots.
        mode_t2w: Switch for T2-weighted image processing.
        mode_t1w: Switch for T1-weighted image processing.
        workdir: Name of the working subdirectory in the output directory.
        out_match_ref: Match the final output volume space FOV and spatial\
            resolution to the reference file.
        do_ceil_out: Apply a ceiling based on the 98%ile value within an\
            automasked volume.
        extra_al_wtmask: Extra weight mask to emphasize specific parts for\
            alignment.
        extra_al_cost: Specify a cost function for 3dAllineate to use (default\
            'lpa').
        extra_al_opts: Extra options for 3dAllineate when applying the warp.
        focus_mask: Input mask to focus processing and alignment.
        focus_by_ss: Make a mask by simply skullstripping input data set.
        remove_inf_sli: Remove a number of slices from the inferior part of the\
            FOV.
        pre_align_center_mass: Pre-align the centers of mass of the volumes.
        pre_center_mass: Pre-recenter input center of mass to (0, 0, 0).
        post_lr_symm: Apply post-alignment left-right symmetrization.
        no_pre_lr_symm: Turn off pre-alignment left-right symmetrization.
        no_clean: Do not remove working directory '__WORKING_axialize_anat'.
        no_qc_view: Turn off default QC image saving/viewing.
        qc_prefix: Provide a prefix for QC outputs separate from the main\
            prefix.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FatProcAxializeAnatOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FAT_PROC_AXIALIZE_ANAT_METADATA)
    cargs = []
    cargs.append("fat_proc_axialize_anat")
    cargs.append("-inset")
    cargs.append(execution.input_file(in_file))
    cargs.append("-refset")
    cargs.append(execution.input_file(ref_file))
    if mode_t2w:
        cargs.append("-mode_t2w")
    if mode_t1w:
        cargs.append("-mode_t1w")
    cargs.append("-prefix")
    cargs.append(prefix)
    if workdir is not None:
        cargs.extend([
            "-workdir",
            workdir
        ])
    if out_match_ref:
        cargs.append("-out_match_ref")
    if do_ceil_out:
        cargs.append("-do_ceil_out")
    if extra_al_wtmask is not None:
        cargs.extend([
            "-extra_al_wtmask",
            execution.input_file(extra_al_wtmask)
        ])
    if extra_al_cost is not None:
        cargs.extend([
            "-extra_al_cost",
            extra_al_cost
        ])
    if extra_al_opts is not None:
        cargs.extend([
            "-extra_al_opts",
            extra_al_opts
        ])
    if focus_mask is not None:
        cargs.extend([
            "-focus_mask",
            execution.input_file(focus_mask)
        ])
    if focus_by_ss:
        cargs.append("-focus_by_ss")
    if remove_inf_sli is not None:
        cargs.extend([
            "-remove_inf_sli",
            str(remove_inf_sli)
        ])
    if pre_align_center_mass:
        cargs.append("-pre_align_center_mass")
    if pre_center_mass:
        cargs.append("-pre_center_mass")
    if post_lr_symm:
        cargs.append("-post_lr_symm")
    if no_pre_lr_symm:
        cargs.append("-no_pre_lr_symm")
    if no_clean:
        cargs.append("-no_clean")
    cargs.append("[QC_ULAY_RANGE_MIN]")
    cargs.append("[QC_ULAY_RANGE_MAX]")
    if no_qc_view:
        cargs.append("-no_qc_view")
    if qc_prefix is not None:
        cargs.extend([
            "-qc_prefix",
            qc_prefix
        ])
    ret = FatProcAxializeAnatOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(prefix + ".nii.gz"),
        working_directory=execution.output_file(workdir) if (workdir is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FAT_PROC_AXIALIZE_ANAT_METADATA",
    "FatProcAxializeAnatOutputs",
    "fat_proc_axialize_anat",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__MAKE_PLUG_DIFF_METADATA = Metadata(
    id="860833daa362c18ad9ef4feed44997f022b01dae.boutiques",
    name="@make_plug_diff",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VMakePlugDiffOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__make_plug_diff(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def v__make_plug_diff(
    vtk_dir: str,
    xm_dir: str,
    afni_src_dir: str,
    afni_bin_dir: str,
    diff_dir: str,
    comments: bool = False,
    linux: bool = False,
    runner: Runner | None = None,
) -> VMakePlugDiffOutputs:
    """
    Compiles AFNI's diffusion plugin.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/@make_plug_diff.html
    
    Args:
        vtk_dir: Directory where vtk is installed.
        xm_dir: Directory where motif is installed.
        afni_src_dir: Full path to AFNI's src/ directory.
        afni_bin_dir: Path, relative to ASRCDIR, to abin.
        diff_dir: Name of directory containing diffusion code.
        comments: Output comments only.
        linux: Flag for doing linuxy things.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VMakePlugDiffOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__MAKE_PLUG_DIFF_METADATA)
    cargs = []
    cargs.append("@make_plug_diff")
    cargs.append("-vtk")
    cargs.extend([
        "-vtk",
        vtk_dir
    ])
    cargs.append("-xm")
    cargs.extend([
        "-xm",
        xm_dir
    ])
    cargs.append("-asrc")
    cargs.extend([
        "-asrc",
        afni_src_dir
    ])
    cargs.append("-abin")
    cargs.extend([
        "-abin",
        afni_bin_dir
    ])
    if comments:
        cargs.append("-comments")
    if linux:
        cargs.append("-linux")
    cargs.extend([
        "-diff",
        diff_dir
    ])
    ret = VMakePlugDiffOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VMakePlugDiffOutputs",
    "V__MAKE_PLUG_DIFF_METADATA",
    "v__make_plug_diff",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

IMCP_METADATA = Metadata(
    id="4c8d6b3e7e8e931e5a4519149846c8c8c3901d6a.boutiques",
    name="imcp",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class ImcpOutputs(typing.NamedTuple):
    """
    Output object returned when calling `imcp(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfiles: OutputPathType
    """Output file or directory"""


def imcp(
    infiles: list[InputPathType],
    output_location: str,
    runner: Runner | None = None,
) -> ImcpOutputs:
    """
    Copy images from one location to another.
    
    Author: Unknown
    
    Args:
        infiles: Input image files (e.g. img1.nii.gz, img2.nii.gz).
        output_location: Output file or directory.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ImcpOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(IMCP_METADATA)
    cargs = []
    cargs.append("imcp")
    cargs.extend([execution.input_file(f) for f in infiles])
    cargs.append(output_location)
    ret = ImcpOutputs(
        root=execution.output_file("."),
        outfiles=execution.output_file(output_location),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "IMCP_METADATA",
    "ImcpOutputs",
    "imcp",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TBSS_SYM_METADATA = Metadata(
    id="e4be653286686df6b0a14dfb1d8ad1dfbc341908.boutiques",
    name="tbss_sym",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class TbssSymOutputs(typing.NamedTuple):
    """
    Output object returned when calling `tbss_sym(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    skeleton_output: OutputPathType
    """Output skeleton image"""


def tbss_sym(
    input_image: InputPathType,
    alternative_4ddata: InputPathType | None = None,
    alternative_skeleton: InputPathType | None = None,
    output_image: InputPathType | None = None,
    help_flag: bool = False,
    debug_flag: bool = False,
    debug2_flag: InputPathType | None = None,
    runner: Runner | None = None,
) -> TbssSymOutputs:
    """
    TBSS Skeletonization tool from FSL.
    
    Author: University of Oxford (Stephen Smith)
    
    URL:
    https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/TBSS/UserGuide#Step_4:_tbss_skeleton
    
    Args:
        input_image: Input image.
        alternative_4ddata: Alternative 4D data (e.g., L1).
        alternative_skeleton: Alternative skeleton.
        output_image: Output skeleton image.
        help_flag: Display help message.
        debug_flag: Enable debugging image outputs.
        debug2_flag: De-project skelpoints points on skeleton back to all_FA\
            space.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TbssSymOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TBSS_SYM_METADATA)
    cargs = []
    cargs.append("tbss_skeleton")
    cargs.extend([
        "-i",
        execution.input_file(input_image)
    ])
    cargs.append("[SKEL_THRESHOLD]")
    cargs.append("[DISTANCE_MAP]")
    cargs.append("[SEARCH_RULE_MASK]")
    cargs.append("[4DDATA]")
    cargs.append("[PROJECTED_4DDATA]")
    if alternative_4ddata is not None:
        cargs.extend([
            "-a",
            execution.input_file(alternative_4ddata)
        ])
    if alternative_skeleton is not None:
        cargs.extend([
            "-s",
            execution.input_file(alternative_skeleton)
        ])
    if output_image is not None:
        cargs.extend([
            "-o",
            execution.input_file(output_image)
        ])
    if help_flag:
        cargs.append("-h")
    if debug_flag:
        cargs.append("-d")
    if debug2_flag is not None:
        cargs.extend([
            "-D",
            execution.input_file(debug2_flag)
        ])
    ret = TbssSymOutputs(
        root=execution.output_file("."),
        skeleton_output=execution.output_file("skeleton_output.nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TBSS_SYM_METADATA",
    "TbssSymOutputs",
    "tbss_sym",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MSM_METADATA = Metadata(
    id="5610be5b5ab6ee9fca03d6770ef7f566c0f406d4.boutiques",
    name="msm",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class MsmOutputs(typing.NamedTuple):
    """
    Output object returned when calling `msm(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Output file from MSM"""


def msm(
    inmesh: InputPathType,
    out: str,
    refmesh: InputPathType | None = None,
    indata: InputPathType | None = None,
    refdata: InputPathType | None = None,
    trans: InputPathType | None = None,
    in_register: InputPathType | None = None,
    inweight: InputPathType | None = None,
    refweight: InputPathType | None = None,
    format_: str | None = None,
    conf: InputPathType | None = None,
    levels: float | None = None,
    smoothout: float | None = None,
    help_: bool = False,
    verbose: bool = False,
    printoptions: bool = False,
    runner: Runner | None = None,
) -> MsmOutputs:
    """
    MSM (Multimodal Surface Matching) is a tool for aligning brain surface scans
    based on their cortical folding patterns or functional/structural data.
    
    Author: Oxford Centre for Functional MRI of the Brain (FMRIB)
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/MSM
    
    Args:
        inmesh: Input mesh (available formats: VTK, ASCII, GIFTI). Needs to be\
            a sphere.
        out: Output basename.
        refmesh: Reference mesh (available formats: VTK, ASCII, GIFTI). Needs\
            to be a sphere. If not included algorithm assumes reference mesh is\
            equivalent input.
        indata: Scalar or multivariate data for input - can be ASCII\
            (.asc,.dpv,.txt) or GIFTI (.func.gii or .shape.gii).
        refdata: Scalar or multivariate data for reference - can be ASCII\
            (.asc,.dpv,.txt) or GIFTI (.func.gii or .shape.gii).
        trans: Transformed source mesh (output of a previous registration). Use\
            this to initialise the current registration.
        in_register: Input mesh at data resolution. Used to resample data onto\
            input mesh if data is supplied at a different resolution. Note this\
            mesh HAS to be in alignment with either the input_mesh of (if supplied)\
            the transformed source mesh. Use with supreme caution.
        inweight: Cost function weighting for input - weights data in these\
            vertices when calculating similarity (ASCII or GIFTI). Can be\
            multivariate provided dimension equals that of data.
        refweight: Cost function weighting for reference - weights data in\
            these vertices when calculating similarity (ASCII or GIFTI). Can be\
            multivariate provided dimension equals that of data.
        format_: Format of output files, can be: GIFTI, VTK, ASCII or ASCII_MAT\
            (for full details of output file formats see MSM wiki).
        conf: Configuration file.
        levels: Number of resolution levels (default = number of resolution\
            levels specified by --opt in config file).
        smoothout: Smooth transformed output with this sigma (default=0).
        help_: Display help message.
        verbose: Switch on diagnostic messages.
        printoptions: Print configuration file options.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MsmOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MSM_METADATA)
    cargs = []
    cargs.append("msm")
    cargs.append(execution.input_file(inmesh))
    cargs.append("--out")
    cargs.append(out)
    if refmesh is not None:
        cargs.extend([
            "--refmesh",
            execution.input_file(refmesh)
        ])
    if indata is not None:
        cargs.extend([
            "--indata",
            execution.input_file(indata)
        ])
    if refdata is not None:
        cargs.extend([
            "--refdata",
            execution.input_file(refdata)
        ])
    if trans is not None:
        cargs.extend([
            "--trans",
            execution.input_file(trans)
        ])
    if in_register is not None:
        cargs.extend([
            "--in_register",
            execution.input_file(in_register)
        ])
    if inweight is not None:
        cargs.extend([
            "--inweight",
            execution.input_file(inweight)
        ])
    if refweight is not None:
        cargs.extend([
            "--refweight",
            execution.input_file(refweight)
        ])
    if format_ is not None:
        cargs.extend([
            "-f",
            format_
        ])
    if conf is not None:
        cargs.extend([
            "--conf",
            execution.input_file(conf)
        ])
    if levels is not None:
        cargs.extend([
            "--levels",
            str(levels)
        ])
    if smoothout is not None:
        cargs.extend([
            "--smoothout",
            str(smoothout)
        ])
    if help_:
        cargs.append("-h")
    if verbose:
        cargs.append("-v")
    if printoptions:
        cargs.append("-p")
    ret = MsmOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(out + "_output.ext"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MSM_METADATA",
    "MsmOutputs",
    "msm",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_FWHMX_METADATA = Metadata(
    id="f47d1eb7ee05c44b288128e5fb15f252f017d16a.boutiques",
    name="3dFWHMx",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dFwhmxOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_fwhmx(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_file: OutputPathType | None
    """Output file containing FWHM/ACF estimates"""
    detrended_dataset: OutputPathType | None
    """Detrended dataset file"""


def v_3d_fwhmx(
    infile: InputPathType,
    mask: InputPathType | None = None,
    automask: bool = False,
    demed: bool = False,
    unif: bool = False,
    detrend: float | None = None,
    detprefix: str | None = None,
    geom: bool = False,
    arith: bool = False,
    combine: bool = False,
    out: str | None = None,
    compat: bool = False,
    acf: str | None = None,
    runner: Runner | None = None,
) -> V3dFwhmxOutputs:
    """
    Compute Full Width at Half Maximum (FWHM) for FMRI datasets using
    AutoCorrelation Function (ACF).
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dFWHMx.html
    
    Args:
        infile: Input dataset.
        mask: Use only voxels that are nonzero in dataset 'mmm'.
        automask: Compute a mask from THIS dataset.
        demed: if the input dataset has more than one sub-brick then subtract\
            the median of each voxel's time series before processing FWHM.
        unif: Normalize each voxel's time series to have the same MAD before\
            processing FWHM, implies -demed.
        detrend: Detrend to order 'q'. If q is not given, the program picks\
            q=NT/30; -detrend disables -demed, and includes -unif.
        detprefix: Save the detrended file into a dataset with prefix 'd'.
        geom: Compute the final estimate as the geometric mean.
        arith: Compute the final estimate as the arithmetic mean.
        combine: Combine the final measurements along each axis into one result.
        out: Write output to file 'ttt' (3 columns of numbers). If not given,\
            the sub-brick outputs are not written. Use '-out -' to write to stdout,\
            if desired.
        compat: Be compatible with the older 3dFWHM.
        acf: Compute the spatial autocorrelation of the data as a function of\
            radius, then fit that to a model and output the model parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dFwhmxOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_FWHMX_METADATA)
    cargs = []
    cargs.append("3dFWHMx")
    if mask is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask)
        ])
    if automask:
        cargs.append("-automask")
    if demed:
        cargs.append("-demed")
    if unif:
        cargs.append("-unif")
    if detrend is not None:
        cargs.extend([
            "-detrend",
            str(detrend)
        ])
    if detprefix is not None:
        cargs.extend([
            "-detprefix",
            detprefix
        ])
    if geom:
        cargs.append("-geom")
    if arith:
        cargs.append("-arith")
    if combine:
        cargs.append("-combine")
    if out is not None:
        cargs.extend([
            "-out",
            out
        ])
    if compat:
        cargs.append("-compat")
    if acf is not None:
        cargs.extend([
            "-acf",
            acf
        ])
    cargs.append(execution.input_file(infile))
    ret = V3dFwhmxOutputs(
        root=execution.output_file("."),
        out_file=execution.output_file(out + ".1D") if (out is not None) else None,
        detrended_dataset=execution.output_file(detprefix + ".nii.gz") if (detprefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dFwhmxOutputs",
    "V_3D_FWHMX_METADATA",
    "v_3d_fwhmx",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRCONVERT_METADATA = Metadata(
    id="bbc290e9da0cf0c8176c0247f73ecbe29ff60e51.boutiques",
    name="mrconvert",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


@dataclasses.dataclass
class MrconvertCoord:
    """
    retain data from the input image only at the coordinates specified in the
    selection along the specified axis. The selection argument expects a number
    sequence, which can also include the 'end' keyword.
    """
    axis: int
    """retain data from the input image only at the coordinates specified in the
    selection along the specified axis. The selection argument expects a number
    sequence, which can also include the 'end' keyword."""
    selection: list[int]
    """retain data from the input image only at the coordinates specified in the
    selection along the specified axis. The selection argument expects a number
    sequence, which can also include the 'end' keyword."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-coord")
        cargs.append(str(self.axis))
        cargs.extend(map(str, self.selection))
        return cargs


@dataclasses.dataclass
class MrconvertClearProperty:
    """
    remove the specified key from the image header altogether.
    """
    key: str
    """remove the specified key from the image header altogether."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-clear_property")
        cargs.append(self.key)
        return cargs


@dataclasses.dataclass
class MrconvertSetProperty:
    """
    set the value of the specified key in the image header.
    """
    key: str
    """set the value of the specified key in the image header."""
    value: str
    """set the value of the specified key in the image header."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-set_property")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


@dataclasses.dataclass
class MrconvertAppendProperty:
    """
    append the given value to the specified key in the image header (this adds
    the value specified as a new line in the header value).
    """
    key: str
    """append the given value to the specified key in the image header (this
    adds the value specified as a new line in the header value)."""
    value: str
    """append the given value to the specified key in the image header (this
    adds the value specified as a new line in the header value)."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-append_property")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


@dataclasses.dataclass
class MrconvertFslgrad:
    """
    Provide the diffusion-weighted gradient scheme used in the acquisition in
    FSL bvecs/bvals format files. If a diffusion gradient scheme is present in
    the input image header, the data provided with this option will be instead
    used.
    """
    bvecs: InputPathType
    """Provide the diffusion-weighted gradient scheme used in the acquisition in
    FSL bvecs/bvals format files. If a diffusion gradient scheme is present in
    the input image header, the data provided with this option will be instead
    used."""
    bvals: InputPathType
    """Provide the diffusion-weighted gradient scheme used in the acquisition in
    FSL bvecs/bvals format files. If a diffusion gradient scheme is present in
    the input image header, the data provided with this option will be instead
    used."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-fslgrad")
        cargs.append(execution.input_file(self.bvecs))
        cargs.append(execution.input_file(self.bvals))
        return cargs


class MrconvertExportGradFslOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrconvertExportGradFsl | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    bvecs_path: OutputPathType
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""
    bvals_path: OutputPathType
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""


@dataclasses.dataclass
class MrconvertExportGradFsl:
    """
    export the diffusion-weighted gradient table to files in FSL (bvecs / bvals)
    format.
    """
    bvecs_path: str
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""
    bvals_path: str
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-export_grad_fsl")
        cargs.append(self.bvecs_path)
        cargs.append(self.bvals_path)
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> MrconvertExportGradFslOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `MrconvertExportGradFslOutputs`).
        """
        ret = MrconvertExportGradFslOutputs(
            root=execution.output_file("."),
            bvecs_path=execution.output_file(self.bvecs_path),
            bvals_path=execution.output_file(self.bvals_path),
        )
        return ret


@dataclasses.dataclass
class MrconvertImportPeEddy:
    """
    import phase-encoding information from an EDDY-style config / index file
    pair.
    """
    config: InputPathType
    """import phase-encoding information from an EDDY-style config / index file
    pair"""
    indices: InputPathType
    """import phase-encoding information from an EDDY-style config / index file
    pair"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-import_pe_eddy")
        cargs.append(execution.input_file(self.config))
        cargs.append(execution.input_file(self.indices))
        return cargs


class MrconvertExportPeEddyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrconvertExportPeEddy | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    config: OutputPathType
    """export phase-encoding information to an EDDY-style config / index file
    pair"""
    indices: OutputPathType
    """export phase-encoding information to an EDDY-style config / index file
    pair"""


@dataclasses.dataclass
class MrconvertExportPeEddy:
    """
    export phase-encoding information to an EDDY-style config / index file pair.
    """
    config: str
    """export phase-encoding information to an EDDY-style config / index file
    pair"""
    indices: str
    """export phase-encoding information to an EDDY-style config / index file
    pair"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-export_pe_eddy")
        cargs.append(self.config)
        cargs.append(self.indices)
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> MrconvertExportPeEddyOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `MrconvertExportPeEddyOutputs`).
        """
        ret = MrconvertExportPeEddyOutputs(
            root=execution.output_file("."),
            config=execution.output_file(self.config),
            indices=execution.output_file(self.indices),
        )
        return ret


@dataclasses.dataclass
class MrconvertConfig:
    """
    temporarily set the value of an MRtrix config file entry.
    """
    key: str
    """temporarily set the value of an MRtrix config file entry."""
    value: str
    """temporarily set the value of an MRtrix config file entry."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-config")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


class MrconvertOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mrconvert(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image."""
    json_export: OutputPathType | None
    """export data from an image header key-value pairs into a JSON file """
    export_grad_mrtrix: OutputPathType | None
    """export the diffusion-weighted gradient table to file in MRtrix format """
    export_pe_table: OutputPathType | None
    """export phase-encoding table to file """
    export_grad_fsl: MrconvertExportGradFslOutputs | None
    """Outputs from `MrconvertExportGradFsl`."""
    export_pe_eddy: MrconvertExportPeEddyOutputs | None
    """Outputs from `MrconvertExportPeEddy`."""


def mrconvert(
    input_: InputPathType,
    output: str,
    coord: list[MrconvertCoord] | None = None,
    vox: list[float] | None = None,
    axes: list[int] | None = None,
    scaling: list[float] | None = None,
    json_import: InputPathType | None = None,
    json_export: str | None = None,
    clear_property: list[MrconvertClearProperty] | None = None,
    set_property: list[MrconvertSetProperty] | None = None,
    append_property: list[MrconvertAppendProperty] | None = None,
    copy_properties: str | None = None,
    strides: str | None = None,
    datatype: str | None = None,
    grad: InputPathType | None = None,
    fslgrad: MrconvertFslgrad | None = None,
    bvalue_scaling: str | None = None,
    export_grad_mrtrix: str | None = None,
    export_grad_fsl: MrconvertExportGradFsl | None = None,
    import_pe_table: InputPathType | None = None,
    import_pe_eddy: MrconvertImportPeEddy | None = None,
    export_pe_table: str | None = None,
    export_pe_eddy: MrconvertExportPeEddy | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrconvertConfig] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrconvertOutputs:
    """
    Perform conversion between different file types and optionally extract a subset
    of the input image.
    
    If used correctly, this program can be a very useful workhorse. In addition
    to converting images between different formats, it can be used to extract
    specific studies from a data set, extract a specific region of interest, or
    flip the images. Some of the possible operations are described in more
    detail below.
    
    Note that for both the -coord and -axes options, indexing starts from 0
    rather than 1. E.g. -coord 3 <#> selects volumes (the fourth dimension) from
    the series; -axes 0,1,2 includes only the three spatial axes in the output
    image.
    
    Additionally, for the second input to the -coord option and the -axes
    option, you can use any valid number sequence in the selection, as well as
    the 'end' keyword (see the main documentation for details); this can be
    particularly useful to select multiple coordinates.
    
    The -vox option is used to change the size of the voxels in the output image
    as reported in the image header; note however that this does not re-sample
    the image based on a new voxel size (that is done using the mrgrid command).
    
    By default, the intensity scaling parameters in the input image header are
    passed through to the output image header when writing to an integer image,
    and reset to 0,1 (i.e. no scaling) for floating-point and binary images.
    Note that the -scaling option will therefore have no effect for
    floating-point or binary output images.
    
    The -axes option specifies which axes from the input image will be used to
    form the output image. This allows the permutation, omission, or addition of
    axes into the output image. The axes should be supplied as a comma-separated
    list of axis indices. If an axis from the input image is to be omitted from
    the output image, it must either already have a size of 1, or a single
    coordinate along that axis must be selected by the user by using the -coord
    option. Examples are provided further below.
    
    The -bvalue_scaling option controls an aspect of the import of diffusion
    gradient tables. When the input diffusion-weighting direction vectors have
    norms that differ substantially from unity, the b-values will be scaled by
    the square of their corresponding vector norm (this is how multi-shell
    acquisitions are frequently achieved on scanner platforms). However in some
    rare instances, the b-values may be correct, despite the vectors not being
    of unit norm (or conversely, the b-values may need to be rescaled even
    though the vectors are close to unit norm). This option allows the user to
    control this operation and override MRrtix3's automatic detection.
    
    References:
    
    .
    
    Author: J-Donald Tournier (jdtournier@gmail.com) and Robert E. Smith
    (robert.smith@florey.edu.au)
    
    URL:
    https://mrtrix.readthedocs.io/en/latest/reference/commands/mrconvert.html
    
    Args:
        input_: the input image.
        output: the output image.
        coord: retain data from the input image only at the coordinates\
            specified in the selection along the specified axis. The selection\
            argument expects a number sequence, which can also include the 'end'\
            keyword.
        vox: change the voxel dimensions reported in the output image header.
        axes: specify the axes from the input image that will be used to form\
            the output image.
        scaling: specify the data scaling parameters used to rescale the\
            intensity values.
        json_import: import data from a JSON file into header key-value pairs.
        json_export: export data from an image header key-value pairs into a\
            JSON file.
        clear_property: remove the specified key from the image header\
            altogether.
        set_property: set the value of the specified key in the image header.
        append_property: append the given value to the specified key in the\
            image header (this adds the value specified as a new line in the header\
            value).
        copy_properties: clear all generic properties and replace with the\
            properties from the image / file specified.
        strides: specify the strides of the output data in memory; either as a\
            comma-separated list of (signed) integers, or as a template image from\
            which the strides shall be extracted and used. The actual strides\
            produced will depend on whether the output image format can support it.
        datatype: specify output image data type. Valid choices are: float32,\
            float32le, float32be, float64, float64le, float64be, int64, uint64,\
            int64le, uint64le, int64be, uint64be, int32, uint32, int32le, uint32le,\
            int32be, uint32be, int16, uint16, int16le, uint16le, int16be, uint16be,\
            cfloat32, cfloat32le, cfloat32be, cfloat64, cfloat64le, cfloat64be,\
            int8, uint8, bit.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvalue_scaling: enable or disable scaling of diffusion b-values by the\
            square of the corresponding DW gradient norm (see Desciption). Valid\
            choices are yes/no, true/false, 0/1 (default: automatic).
        export_grad_mrtrix: export the diffusion-weighted gradient table to\
            file in MRtrix format.
        export_grad_fsl: export the diffusion-weighted gradient table to files\
            in FSL (bvecs / bvals) format.
        import_pe_table: import a phase-encoding table from file.
        import_pe_eddy: import phase-encoding information from an EDDY-style\
            config / index file pair.
        export_pe_table: export phase-encoding table to file.
        export_pe_eddy: export phase-encoding information to an EDDY-style\
            config / index file pair.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrconvertOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRCONVERT_METADATA)
    cargs = []
    cargs.append("mrconvert")
    if coord is not None:
        cargs.extend([a for c in [s.run(execution) for s in coord] for a in c])
    if vox is not None:
        cargs.extend([
            "-vox",
            *map(str, vox)
        ])
    if axes is not None:
        cargs.extend([
            "-axes",
            ",".join(map(str, axes))
        ])
    if scaling is not None:
        cargs.extend([
            "-scaling",
            *map(str, scaling)
        ])
    if json_import is not None:
        cargs.extend([
            "-json_import",
            execution.input_file(json_import)
        ])
    if json_export is not None:
        cargs.extend([
            "-json_export",
            json_export
        ])
    if clear_property is not None:
        cargs.extend([a for c in [s.run(execution) for s in clear_property] for a in c])
    if set_property is not None:
        cargs.extend([a for c in [s.run(execution) for s in set_property] for a in c])
    if append_property is not None:
        cargs.extend([a for c in [s.run(execution) for s in append_property] for a in c])
    if copy_properties is not None:
        cargs.extend([
            "-copy_properties",
            copy_properties
        ])
    if strides is not None:
        cargs.extend([
            "-strides",
            strides
        ])
    if datatype is not None:
        cargs.extend([
            "-datatype",
            datatype
        ])
    if grad is not None:
        cargs.extend([
            "-grad",
            execution.input_file(grad)
        ])
    if fslgrad is not None:
        cargs.extend(fslgrad.run(execution))
    if bvalue_scaling is not None:
        cargs.extend([
            "-bvalue_scaling",
            bvalue_scaling
        ])
    if export_grad_mrtrix is not None:
        cargs.extend([
            "-export_grad_mrtrix",
            export_grad_mrtrix
        ])
    if export_grad_fsl is not None:
        cargs.extend(export_grad_fsl.run(execution))
    if import_pe_table is not None:
        cargs.extend([
            "-import_pe_table",
            execution.input_file(import_pe_table)
        ])
    if import_pe_eddy is not None:
        cargs.extend(import_pe_eddy.run(execution))
    if export_pe_table is not None:
        cargs.extend([
            "-export_pe_table",
            export_pe_table
        ])
    if export_pe_eddy is not None:
        cargs.extend(export_pe_eddy.run(execution))
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([a for c in [s.run(execution) for s in config] for a in c])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    cargs.append(execution.input_file(input_))
    cargs.append(output)
    ret = MrconvertOutputs(
        root=execution.output_file("."),
        output=execution.output_file(output),
        json_export=execution.output_file(json_export) if (json_export is not None) else None,
        export_grad_mrtrix=execution.output_file(export_grad_mrtrix) if (export_grad_mrtrix is not None) else None,
        export_pe_table=execution.output_file(export_pe_table) if (export_pe_table is not None) else None,
        export_grad_fsl=export_grad_fsl.outputs(execution) if export_grad_fsl else None,
        export_pe_eddy=export_pe_eddy.outputs(execution) if export_pe_eddy else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRCONVERT_METADATA",
    "MrconvertAppendProperty",
    "MrconvertClearProperty",
    "MrconvertConfig",
    "MrconvertCoord",
    "MrconvertExportGradFsl",
    "MrconvertExportGradFslOutputs",
    "MrconvertExportPeEddy",
    "MrconvertExportPeEddyOutputs",
    "MrconvertFslgrad",
    "MrconvertImportPeEddy",
    "MrconvertOutputs",
    "MrconvertSetProperty",
    "mrconvert",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

DIRMERGE_METADATA = Metadata(
    id="3ce611683ba7275413635b52cfd65e02dc5a3ec7.boutiques",
    name="dirmerge",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


@dataclasses.dataclass
class DirmergeConfig:
    """
    temporarily set the value of an MRtrix config file entry.
    """
    key: str
    """temporarily set the value of an MRtrix config file entry."""
    value: str
    """temporarily set the value of an MRtrix config file entry."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-config")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


class DirmergeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `dirmerge(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out: OutputPathType
    """the output directions file, with each row listing the X Y Z gradient
    directions, the b-value, and an index representing the phase encode
    direction"""


def dirmerge(
    subsets: int,
    bvalue_files: list[str],
    out: str,
    unipolar_weight: float | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DirmergeConfig] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> DirmergeOutputs:
    """
    Splice / merge multiple sets of directions in such a way as to maintain
    near-optimality upon truncation.
    
    
    
    References:
    
    .
    
    Author: J-Donald Tournier (jdtournier@gmail.com)
    
    URL:
    https://mrtrix.readthedocs.io/en/latest/reference/commands/dirmerge.html
    
    Args:
        subsets: the number of subsets (eg. phase encoding directions) per\
            b-value.
        bvalue_files: the b-value and sets of corresponding files, in order.
        out: the output directions file, with each row listing the X Y Z\
            gradient directions, the b-value, and an index representing the phase\
            encode direction.
        unipolar_weight: set the weight given to the unipolar electrostatic\
            repulsion model compared to the bipolar model (default: 0.2).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DirmergeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(DIRMERGE_METADATA)
    cargs = []
    cargs.append("dirmerge")
    if unipolar_weight is not None:
        cargs.extend([
            "-unipolar_weight",
            str(unipolar_weight)
        ])
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([a for c in [s.run(execution) for s in config] for a in c])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    cargs.append(str(subsets))
    cargs.extend(bvalue_files)
    cargs.append(out)
    ret = DirmergeOutputs(
        root=execution.output_file("."),
        out=execution.output_file(out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "DIRMERGE_METADATA",
    "DirmergeConfig",
    "DirmergeOutputs",
    "dirmerge",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

QBOOT_METADATA = Metadata(
    id="719956ac6e06444e87fb8bebc3ba3438af528fd3.boutiques",
    name="qboot",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class QbootOutputs(typing.NamedTuple):
    """
    Output object returned when calling `qboot(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_files: OutputPathType
    """Output files saved in the specified log directory"""


def qboot(
    data_file: InputPathType,
    mask_file: InputPathType,
    bvecs_file: InputPathType,
    bvals_file: InputPathType,
    log_dir: str | None = None,
    forcedir_flag: bool = False,
    q_file: InputPathType | None = None,
    model_type: int | None = None,
    lmax_order: int | None = None,
    npeaks: int | None = None,
    threshold: float | None = None,
    num_samples: int | None = None,
    lambda_param: float | None = None,
    delta_param: float | None = None,
    alpha_param: float | None = None,
    seed_param: int | None = None,
    gfa_flag: bool = False,
    savecoeff_flag: bool = False,
    savemeancoeff_flag: bool = False,
    verbose_flag: bool = False,
    help_flag: bool = False,
    runner: Runner | None = None,
) -> QbootOutputs:
    """
    Tool for computing q-ball ODFs using bootstrap samples.
    
    Author: Oxford Centre for Functional MRI of the Brain (FMRIB)
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/
    
    Args:
        data_file: Data file.
        mask_file: Mask file.
        bvecs_file: b vectors file.
        bvals_file: b values file.
        log_dir: Output directory (default is logdir).
        forcedir_flag: Use the actual directory name given - i.e. don't add +\
            to make a new directory.
        q_file: File provided with multi-shell data. Indicates the number of\
            directions for each shell.
        model_type: Which model to use. 1=Tuch's ODFs, 2=CSA ODFs (default),\
            3=multi-shell CSA ODFs.
        lmax_order: Maximum spherical harmonic order employed (must be even,\
            default=4).
        npeaks: Maximum number of ODF peaks to be detected (default 2).
        threshold: Minimum threshold for a local maxima to be considered an ODF\
            peak. Expressed as a fraction of the maximum ODF value (default 0.4).
        num_samples: Number of bootstrap samples (default is 50).
        lambda_param: Laplace-Beltrami regularization parameter (default is 0).
        delta_param: Signal attenuation regularization parameter for models=2,3\
            (default is 0.01).
        alpha_param: Laplacian sharpening parameter for model=1 (default is 0,\
            should be smaller than 1).
        seed_param: Seed for pseudo-random number generator.
        gfa_flag: Compute a generalised FA, using the mean ODF in each voxel.
        savecoeff_flag: Save the ODF coefficients instead of the peaks.\
            WARNING: These can be huge files, please use a few bootstrap samples\
            and a low lmax!.
        savemeancoeff_flag: Save the mean ODF coefficients across all samples.
        verbose_flag: Switch on diagnostic messages.
        help_flag: Display this help message.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `QbootOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(QBOOT_METADATA)
    cargs = []
    cargs.append("qboot")
    cargs.extend([
        "-k",
        execution.input_file(data_file)
    ])
    cargs.extend([
        "-m",
        execution.input_file(mask_file)
    ])
    cargs.extend([
        "-r",
        execution.input_file(bvecs_file)
    ])
    cargs.extend([
        "-b",
        execution.input_file(bvals_file)
    ])
    if log_dir is not None:
        cargs.extend([
            "--ld",
            log_dir
        ])
    if forcedir_flag:
        cargs.append("--forcedir")
    if q_file is not None:
        cargs.extend([
            "--q",
            execution.input_file(q_file)
        ])
    if model_type is not None:
        cargs.extend([
            "--model",
            str(model_type)
        ])
    if lmax_order is not None:
        cargs.extend([
            "--lmax",
            str(lmax_order)
        ])
    if npeaks is not None:
        cargs.extend([
            "--npeaks",
            str(npeaks)
        ])
    if threshold is not None:
        cargs.extend([
            "--thr",
            str(threshold)
        ])
    if num_samples is not None:
        cargs.extend([
            "--ns",
            str(num_samples)
        ])
    if lambda_param is not None:
        cargs.extend([
            "--lambda",
            str(lambda_param)
        ])
    if delta_param is not None:
        cargs.extend([
            "--delta",
            str(delta_param)
        ])
    if alpha_param is not None:
        cargs.extend([
            "--alpha",
            str(alpha_param)
        ])
    if seed_param is not None:
        cargs.extend([
            "--seed",
            str(seed_param)
        ])
    if gfa_flag:
        cargs.append("--gfa")
    if savecoeff_flag:
        cargs.append("--savecoeff")
    if savemeancoeff_flag:
        cargs.append("--savemeancoeff")
    if verbose_flag:
        cargs.append("-V")
    if help_flag:
        cargs.append("-h")
    ret = QbootOutputs(
        root=execution.output_file("."),
        output_files=execution.output_file("logdir/*"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "QBOOT_METADATA",
    "QbootOutputs",
    "qboot",
]
